# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.


"""Rotating file log handler for writing logs to the file system.

**Configuration**

======================== ======================================================
`LOGGING_FS_BACKUPCOUNT` Number of files to keep. **Default:** ``5``.
`LOGGING_FS_MAXBYTES`    Max file size in bytes.  **Default:** ``104857600``
                         (100 MB).
`LOGGING_FS_LEVEL`       Log level threshold for handler. **Default:**
                         ``WARNING``.
======================== ======================================================
"""

from __future__ import absolute_import

import logging
import os
from logging.handlers import RotatingFileHandler


def setup_app(app):
    """Filesystem logging handler."""
    app.config.setdefault('LOGGING_FS_BACKUPCOUNT', 5)
    app.config.setdefault('LOGGING_FS_MAXBYTES', 104857600)  # 100mb
    app.config.setdefault(
        'LOGGING_FS_LEVEL',
        'DEBUG' if app.debug else 'WARNING'
    )

    # Create log directory if it does not exists
    try:
        os.makedirs(
            os.path.join(app.instance_path, app.config.get('CFG_LOGDIR', ''))
        )
    except Exception:
        pass

    handler = RotatingFileHandler(
        os.path.join(
            app.instance_path,
            app.config.get('CFG_LOGDIR', ''),
            app.logger_name + '.log'
        ),
        backupCount=app.config['LOGGING_FS_BACKUPCOUNT'],
        maxBytes=app.config['LOGGING_FS_MAXBYTES']
    )

    handler.setFormatter(logging.Formatter(
        '%(asctime)s %(levelname)s: %(message)s '
        '[in %(pathname)s:%(lineno)d]'
    ))

    handler.setLevel(app.config['LOGGING_FS_LEVEL'])

    # Add handler to application logger
    app.logger.addHandler(handler)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2011, 2012, 2013, 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Platform-independent GUID type."""

import uuid

from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.types import CHAR, TypeDecorator


class GUID(TypeDecorator):

    """Platform-independent GUID type.

    Uses Postgresql's :class:`~sqlalchemy.dialects.postgresql.UUID`
    type, otherwise uses :class:`~sqlalchemy.types.CHAR` (32),
    storing as stringified hex values.
    """

    impl = CHAR

    def load_dialect_impl(self, dialect):
        """Load dialect dependent implementation of UUID."""
        if dialect.name == 'postgresql':
            return dialect.type_descriptor(UUID())
        else:
            return dialect.type_descriptor(CHAR(32))

    def process_bind_param(self, value, dialect):
        """Create UUID representation for given dialect."""
        if value is None:
            return value
        elif dialect.name == 'postgresql':
            return str(value)
        else:
            if not isinstance(value, uuid.UUID):
                return "%.32x" % uuid.UUID(value)
            else:
                # hexstring
                return "%.32x" % value

    def process_result_value(self, value, dialect):
        """Create UUID instance."""
        if value is None:
            return value
        else:
            return uuid.UUID(value)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2013, 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""
Flask-Admin support in Invenio
------------------------------

Please see http://flask-admin.readthedocs.org/en/latest/quickstart/ prior to
reading this documentation, to understand how Flask-Admin works.

Flask admin allows you to easily create web administration interfaces for your
SQLAlchemy models. This extension takes care of using Blueprint as base class
for the admin views.

By default this extension will look for invenio.<name>_admin modules and call
the method register_admin(app, admin) in each module to allow to register its
administration views.

By default all view are restricted to super users only. This can be changed via
the acc_<action>_action class variables.

Usage example - create a file called <module>_admin.py::

    from invenio_ext.admin import InvenioModelView
    from invenio_ext.sqlalchemy import db
    from invenio.<module>_models import MyModel

    class MyModelAdmin(InvenioModelView):
        acc_edit_action = 'cfgmymodel'

        _can_create = False
        _can_edit = True
        _can_delete = False

        # ... Flaks-Admin options ...
        # column_list = ( ... )

        def __init__(self, model, session, **kwargs):
            super(MyModelAdmin, self).__init__(model, session, **kwargs)

    def register_admin(app, admin):
        admin.add_view(MyModelAdmin(MyModel, db.session, name='My model',
                                    category="My Category"))

Admin UI skins
~~~~~~~~~~~~~~

AdminLTE provides several different skins, please see
https://almsaeedstudio.com/themes/AdminLTE/documentation/index.html#layout.

A global variable `ADMIN_UI_SKIN` is defined and is set to `skin-blue` as
default. To change the skin, just edit the value of the variable to one of
the provided skins.
"""

from __future__ import absolute_import

from flask_admin import Admin

from flask_registry import ModuleAutoDiscoveryRegistry

from .views import AdminIndexView


class AdminDiscoveryRegistry(ModuleAutoDiscoveryRegistry):

    """Utility method."""

    setup_func_name = 'register_admin'

    def __init__(self, *args, **kwargs):
        self.admin = kwargs.pop('admin', None)
        super(AdminDiscoveryRegistry, self).__init__(*args, **kwargs)

    def register(self, module, *args, **kwargs):
        super(AdminDiscoveryRegistry, self).register(
            module, self.app, self.admin, *args, **kwargs
        )


def setup_app(app):
    """Register all administration views with the Flask application."""
    app.config.setdefault("ADMIN_NAME", "Invenio")
    app.config.setdefault("ADMIN_UI_SKIN", "skin-blue")

    # Initialize app
    admin = Admin(
        name=app.config['ADMIN_NAME'],
        index_view=AdminIndexView(
            menu_icon_type='fa',
            menu_icon_value='fa-home'),
        base_template="admin_base.html",
        template_mode='bootstrap3'
    )

    # TODO remove when 1.2.1 is released
    if not hasattr(admin, 'category_icon_classes'):
        admin.category_icon_classes = {}

    admin.init_app(app)

    # Create registry and run discovery
    app.extensions['registry']['admin'] = AdminDiscoveryRegistry(
        'admin', app=app, with_setup=True, admin=admin
    )



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Fixtures extension."""

from __future__ import print_function

from .registry import fixtures


def load_fixtures(sender, yes_i_know=False, drop=True, **kwargs):
    """Load fixtures.

    Loads classes found in 'packages' to the database. Names of the fixture
    classes should end with 'Data' suffix.

    :param packages: packages with fixture classes to load
    :param truncate_tables_first: if True truncates tables before loading
        the fixtures
    """
    from invenio_ext.sqlalchemy import db, models
    from fixture import SQLAlchemyFixture

    # Load SQLAlchemy models.
    list(models)
    models = dict((m.__name__ + 'Data', m) for m in db.Model.__subclasses__())

    missing = set(fixtures.keys()) - set(models.keys())
    if len(missing):
        raise Exception(
            'Cannot match models for the following fixtures classes {0}'.format(
                missing
            ))
    print(">>> There are", len(fixtures.keys()), "tables to be loaded.")
    SQLAlchemyFixture(
        env=models, engine=db.metadata.bind, session=db.session
    ).data(*fixtures.values()).setup()
    db.session.commit()


def fixture_dump(sender, **kwargs):
    """Dump fixtures."""
    print('ERROR: This feature is not implemented inside fixtures.')


def setup_app(app):
    """Set up the extension for the given app."""
    # Subscribe to database post create command
    from invenio_base import signals
    from invenio_base.scripts.database import create, recreate, dump
    signals.post_command.connect(load_fixtures, sender=create)
    signals.post_command.connect(load_fixtures, sender=recreate)
    signals.post_command.connect(fixture_dump, sender=dump)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2012, 2013, 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Additional decorator for extending template context with new objects."""

import six
from flask import current_app, g, has_app_context, request
from werkzeug.utils import cached_property


def register_template_context_processor(f):
    """Register globally the context processor."""
    g._template_context_processor.append(f)


class template_args(object):

    """Register a context processor function for given endpoint.

    If you need to pass some extra parameters to the template,
    you can override particular template context of any view function.
    Decorated function is executed for **every** single ``render_template``
    executed within you view. For any execution heavy functions use
    caching per request.

    These arguments are local for this request and will be discarded
    in the next request.

    Any value passed through ``_invenio_template_args`` will override whatever
    parent view function passed to the template.

    Example of usage in an extension:

    .. code-block:: python

        def setup_app(app):

            @template_args('collections.index', app=app)
            def foo():
                return dict(foo='bar')


    Example of usage in an overlay ``views.py``:

    .. code-block:: python

        from invenio_collections.views.collections import index

        @template_args(index)
        def bar():
            return {'baz': 'bar'}

    If you define endpoint as string then ``template_args`` should only be
    called from an extensions ``setup_app``.

    .. note::

        Make sure that each ``@template_args`` is called (initialized)
        **only** once.

    """

    def __init__(self, endpoint, app=None):
        """Initialize decorator."""
        self._endpoint = endpoint
        self._app = app

    @cached_property
    def endpoint(self):
        """Return view function for given endpoint."""
        if isinstance(self._endpoint, six.string_types):
            return self.app.view_functions[self._endpoint]
        return self._endpoint

    @cached_property
    def app(self):
        """Return ``app`` from constructor or ``current_app``."""
        if self._app is None and has_app_context():
            return current_app._get_current_object()
        if self._app is None:
            raise Exception('Application context or app argument needed.')
        return self._app

    def _register(self, func):
        """Register a context processor function."""
        if not hasattr(self.endpoint, '_invenio_template_args'):
            setattr(self.endpoint, '_invenio_template_args', [])
        self.endpoint._invenio_template_args.append(func)

    def __call__(self, func):
        """Decorator."""
        if isinstance(self._endpoint, six.string_types):
            @self.app.before_first_request
            def appender():
                self._register(func)
        else:
            self._register(func)


def setup_app(app):
    """Initialize template context processor extension."""
    @app.before_request
    def reset_template_context():
        """Reset custom template context buffer."""
        g._template_context_processor = []

    @app.context_processor
    def inject_template_context():
        """Update `Jinja2` context by dynamic context processors."""
        context = {}
        for func in getattr(g, '_template_context_processor', []):
            context.update(func())

        # used by ``template_args`` decorator.
        endpoint = current_app.view_functions.get(request.endpoint)
        for func in getattr(endpoint, '_invenio_template_args', []):
            context.update(func())

        reset_template_context()
        return context

    return app



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2013, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""
    invenio_ext.sslify
    ------------------
     This module provides initialization and configuration for
     `flask_sslify` module.
"""

from flask import session, request, current_app
from .wrappers import SSLify
from .decorators import ssl_required


def setup_app(app):
    CFG_SITE_SECURE_URL = app.config.get('CFG_SITE_SECURE_URL')
    CFG_SITE_URL = app.config.get('CFG_SITE_URL')
    app.config['CFG_HAS_HTTPS_SUPPORT'] = CFG_HAS_HTTPS_SUPPORT = \
        CFG_SITE_SECURE_URL.startswith("https://")
    app.config['CFG_FULL_HTTPS'] = CFG_FULL_HTTPS = \
        CFG_SITE_URL.lower().startswith("https://")

    if CFG_HAS_HTTPS_SUPPORT:
        # Makes request always run over HTTPS.
        _sslify = SSLify(app)

        if not CFG_FULL_HTTPS:
            @_sslify.criteria_handler
            def criteria():
                """Extends criteria when to stay on HTTP site."""
                _force_https = False
                if request.blueprint in current_app.blueprints:
                    blueprint = current_app.blueprints[request.blueprint]
                    _force_https = getattr(blueprint, '_force_https', False)

                view_func = current_app.view_functions.get(request.endpoint)
                if view_func is not None and hasattr(
                        view_func, '_force_https'):
                    _force_https = view_func._force_https

                return not (_force_https or session.need_https())

    return app

__all__ = ['setup_app', 'ssl_required']



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2011, 2012, 2013, 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Example implementation of SQLAlchemy session model backend."""

from datetime import datetime

from flask_login import current_user

from invenio_ext.sqlalchemy import db


class Session(db.Model):

    """Represent Session record."""

    __tablename__ = 'session'
    session_key = db.Column(db.String(32), nullable=False,
                            server_default='', primary_key=True)
    session_expiry = db.Column(db.DateTime, nullable=True, index=True)
    session_object = db.Column(db.LargeBinary, nullable=True)
    uid = db.Column(db.Integer(15, unsigned=True), nullable=False, index=True)

    def get_session(self, name, expired=False):
        """Return an instance of :class:`Session`."""
        where = Session.session_key == name
        if expired:
            where = db.and_(
                where, Session.session_expiry >= db.func.current_timestamp())
        return self.query.filter(where).one()

    def set_session(self, name, value, timeout=None):
        """Store value in database."""
        uid = current_user.get_id()
        session_expiry = datetime.utcnow() + timeout
        return Session(session_key=name,
                       session_object=value,
                       session_expiry=session_expiry,
                       uid=uid)



# -*- coding: utf-8 -*-
# This file is part of Invenio.
# Copyright (C) 2013, 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Customization of default Flask Jinja2 template loader.

By default the Flask Jinja2 template loader is not aware of the order of
Blueprints as defined by the PACKAGES configuration variable.
"""

from distutils.version import LooseVersion

from flask import __version__ as flask_version
from flask.templating import DispatchingJinjaLoader

# Flask 1.0 changes return value of _iter_loaders so for compatibility with
# both Flask 0.10 and 1.0 we here check the version.
# See Flask commit bafc13981002dee4610234c7c97ac176766181c1
IS_FLASK_1_0 = LooseVersion(flask_version) >= LooseVersion("0.11-dev")

try:
    # Deprecated in Flask commit 817b72d484d353800d907b3580c899314bf7f3c6
    from flask.templating import blueprint_is_module
except ImportError:
    blueprint_is_module = lambda blueprint: False


class OrderAwareDispatchingJinjaLoader(DispatchingJinjaLoader):

    """Order aware dispatching Jinja loader."""

    def _iter_loaders(self, template):
        for blueprint in self.app.extensions['registry']['blueprints']:
            if blueprint_is_module(blueprint):
                continue
            loader = blueprint.jinja_loader
            if loader is not None:
                if IS_FLASK_1_0:
                    yield blueprint, loader
                else:
                    yield loader, template



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Views for Jasmine test runner."""

from __future__ import unicode_literals

from flask import Blueprint, abort, render_template, send_file, url_for

from .registry import specs

blueprint = Blueprint(
    'jasmine',
    __name__,
    url_prefix='/jasmine',
    static_folder='static',
    template_folder='templates',
)


@blueprint.route('/specrunner/', methods=['GET'])
def specrunner():
    """Render Jasmine test runner page."""
    modules = sorted([url_for('jasmine.spec', specpath=x)
                      for x in specs.keys() if x.endswith(".spec.js")])
    return render_template('jasmine/specrunner.html', modules=modules)


@blueprint.route('/spec/<path:specpath>', methods=['GET'])
def spec(specpath):
    """Send a single spec file."""
    if specpath not in specs:
        abort(404)

    mimetype = "text/html" if specpath.endswith(".html") else \
               "application/javascript"

    return send_file(
        specs[specpath],
        mimetype=mimetype,
        conditional=False,
        add_etags=False,
    )



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Mixer extension."""

from __future__ import absolute_import, print_function

import errno
import json
import logging
import os
import sys

from mixer.backend.flask import Mixer as _Mixer

from invenio_ext.sqlalchemy import db, models

from .registry import mixers


class InvalidMixerRequest(Exception):

    """Default Mixer exception."""

    pass


class JSONBlender(object):

    """Default blender.

    Uses json files as input for the values of the sqlalchemy objects
    providing a way to load and dump data from them.
    """

    class Dump(object):

        def __init__(self, scheme):
            self.__scheme = scheme
            self.__fixtures = []
            # If the file does not exit try to create it!
            directory = os.path.dirname(self.__scheme.__source__)
            try:
                os.makedirs(os.path.dirname(directory))
            except OSError as e:
                if e.errno == errno.EEXIST and os.path.isdir(directory):
                    pass
                else:
                    raise
            self.__source = open(self.__scheme.__source__, 'w+')

        def __enter__(self):
            return self.dump

        def __exit__(self, type, value, traceback):
            json.dump(self.__fixtures, self.__source,
                      indent=4, sort_keys=True, separators=(',', ': '))
            self.__source.close()

        def dump(self, obj):
            """Track all the objects to be dumped on exit."""
            self.__fixtures.append(obj)

    class Load(object):

        def __init__(self, scheme, drop=False):
            self.__scheme = scheme
            self.__source = open(self.__scheme.__source__, 'r')
            if drop:
                self.__scheme.query.delete()

        def __enter__(self):
            return self.load

        def __exit__(self, type, value, traceback):
            self.__source.close()

        def load(self):
            """Retrieve one object from the json input."""
            for obj in json.load(self.__source):
                yield obj


class MixerMeta(type):

    """Meta class for the mixers.

    Sets the default value of several class arguments if not set already:

    * `__field__`: Name of the fields from the model to be load and dump using
    the mixer. By default all the fields from the model.
    * `__source__`: path of the json file. By default the path of the current
    mixer class `/sources/tablename.json`.
    * `__blender__`: Blender class to use. By default `JSONBlender`.
    """

    def __new__(mcs, name, bases, dict_):
        if '__model__' not in dict_:
            raise InvalidMixerRequest(
                'Class %s does not have a __model__ specified and does not '
                'inherit from an existing Model-Mixer class')

        if '__fields__' not in dict_:
            dict_['__fields__'] = tuple(
                c.name for c in dict_['__model__'].__table__.columns)

        if '__source__' not in dict_:
            path = os.path.dirname(sys.modules[dict_['__module__']].__file__)
            dict_['__source__'] = os.path.join(
                path, 'sources',
                '%s.json' % (dict_['__model__'].__tablename__, ))

        if '__blender__' not in dict_:
            dict_['__blender__'] = JSONBlender

        return super(MixerMeta, mcs).__new__(mcs, name, bases, dict_)


class Mixer(_Mixer):

    """Custom mixer to use external files as fixtures and not only random."""

    def __init__(self, fake=True, factory=None, loglevel=logging.WARN,
                 silence=False, **params):
        super(
            Mixer,
            self).__init__(
            fake=fake,
            factory=factory,
            loglevel=loglevel,
            silence=silence,
            commit=False)

    def blend(self, scheme, drop=True, **values):
        """Blend scheme object.

        :param scheme: A mixer object.
        :param drop: If true the table that holds the sheme object will be
            dropped.
        :param values: Default values to be use for some fields despite of the
            content of the json file.

        :return: List of objects saved to the data base.
        """
        result = []

        if drop:
            scheme.__model__.query.delete()

        with scheme.__blender__.Load(scheme) as load:
            for fixture_values in load():
                fixture_values.update(values)
                result.append(
                    super(Mixer, self).blend(scheme.__model__,
                                             **fixture_values)
                )

        return result

    def unblend(self, scheme, output_file=None, *criterion):
        """Unblend the content of the data base.

        :param scheme: A mixer object.
        :param output_file: If specified it will be used as output file.
        :param criterion: SQLAlchemy criteria to be use to fetch all the data
            from the database for the current scheme object.
        """
        if output_file:
            scheme.__source__ = output_file
        with scheme.__blender__.Dump(scheme) as dump:
            for row in scheme.__model__.query.filter(*criterion):
                dump(dict((key, value) for (key, value) in row.todict()
                          if key in scheme.__fields__))


mixer = Mixer()
"""Default instance."""


def blend_all(sender, yes_i_know=False, drop=True, **kwargs):
    """Blend all the possible models found in the package.

    :param drop: If `True` delete the previous data.
    """
    print('>>> Mixer: Blending DB')

    # Load all models
    list(models)

    for table in db.metadata.sorted_tables:
        try:
            result = mixer.blend(mixers[table.name], drop)
            db.session.add_all(result)
            db.session.commit()
        except KeyError:
            # There is no mixer for this table
            pass


def unblend_all(sender, **kwargs):
    """Unblend all the possible models found in the package."""
    print('>>> Mixer: Unblending DB')

    # Load all models
    list(models)

    for table in db.metadata.sorted_tables:
        try:
            mixer.unblend(mixers[table.name])
        except KeyError:
            # There is no mixer for this table
            pass


def setup_app(app):
    """Set up the extension for the given app."""
    mixer.init_app(app)
    # Subscribe to database post create and recreate command
    from invenio_base import signals
    from invenio_base.scripts.database import create, recreate, dump
    signals.post_command.connect(blend_all, sender=create)
    signals.post_command.connect(blend_all, sender=recreate)
    signals.post_command.connect(unblend_all, sender=dump)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""A class to test Restful APIs."""

import json

import six
from flask import url_for

from invenio_testing import InvenioTestCase


class APITestCase(InvenioTestCase):

    """API unit test base class."""

    def __init__(self, *args, **kwargs):
        """Set instance variables."""
        super(APITestCase, self).__init__(*args, **kwargs)
        self.accesstoken = dict()

    def create_oauth_token(self, user_id, scopes, is_internal=True):
        """Create an OAuth personal access_token."""
        # Create a personal access token as well.
        from invenio_oauth2server.models import Token
        self.accesstoken[user_id] = Token.create_personal(
            'test-personal-%s' % user_id,
            user_id,
            scopes=scopes,
            is_internal=is_internal
        ).access_token

    def remove_oauth_token(self):
        """Remove oauth tokens."""
        from invenio_ext.sqlalchemy import db
        for t in six.itervalues(self.accesstoken):
            if t:
                from invenio_oauth2server.models import Token
                t = Token.query.filter_by(access_token=t).first()
                if t:
                    db.session.delete(t)
        db.session.commit()
        self.accesstoken = dict()

    def get(self, *args, **kwargs):
        """See APITestCase.make_request()."""
        return self.make_request(self.client.get, *args, **kwargs)

    def head(self, *args, **kwargs):
        """See APITestCase.make_request()."""
        return self.make_request(self.client.head, *args, **kwargs)

    def patch(self, *args, **kwargs):
        """See APITestCase.make_request()."""
        return self.make_request(self.client.patch, *args, **kwargs)

    def options(self, *args, **kwargs):
        """See APITestCase.make_request()."""
        return self.make_request(self.client.options, *args, **kwargs)

    def post(self, *args, **kwargs):
        """See APITestCase.make_request()."""
        return self.make_request(self.client.post, *args, **kwargs)

    def put(self, *args, **kwargs):
        """See APITestCase.make_request()."""
        return self.make_request(self.client.put, *args, **kwargs)

    def delete(self, *args, **kwargs):
        """See APITestCase.make_request()."""
        return self.make_request(self.client.delete, *args, **kwargs)

    def make_request(self, client_func, endpoint, urlargs={}, data=None,
                     is_json=True, code=None, headers=None,
                     follow_redirects=False, user_id=None):
        """Make a request to the API endpoint.

        Ensures request looks like they arrive on CFG_SITE_SECURE_URL.
        That header "Contet-Type: application/json" is added if the parameter
        is_json is True

        :param endpoint: Endpoint passed to url_for.
        :param urlargs: Keyword args passed to url_for
        :param data: Request body, either as a dictionary if ``is_json`` is
            True, or as a string if ``is_json`` is False
        :param headers: List of headers for the request
        :param code: Assert response status code
        :param follow_redirects: Whether to follow redirects.
        """
        if headers is None:
            headers = [('content-type', 'application/json')] if is_json else []

        if data is not None:
            request_args = dict(
                data=json.dumps(data) if is_json else data,
                headers=headers,
            )
        else:
            request_args = {}

        if user_id is None:
            if len(self.accesstoken) == 1:
                user_id = self.accesstoken.keys()[0]
        assert user_id is not None, "Please provide a user_id argument."

        if user_id in self.accesstoken:
            urlargs['access_token'] = self.accesstoken[user_id]

        url = url_for(endpoint, **urlargs)
        response = client_func(
            url,
            base_url=self.app.config['CFG_SITE_SECURE_URL'],
            follow_redirects=follow_redirects,
            **request_args
        )
        if code is not None:
            self.assertStatus(response, code)
        return response



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2013, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""
Invenio Admin mail backend.  send_email() will send emails only to
CFG_SITE_ADMIN_EMAIL.
"""

from flask import current_app


def adminonly_class(Backend):
    """Admin only mail backend class factory."""

    class _Mail(Backend):

        def send_messages(self, email_messages):
            def process_message(m):
                m.body = """
#--------------------------------------------------------------
#This message would have been sent to the following recipients:
#%s
#--------------------------------------------------------------
#%s""" % (','.join(m.recipients()), m.body)
                m.to = [current_app.config['CFG_SITE_ADMIN_EMAIL']]
                m.cc = []
                m.bcc = []
                if 'Cc' in m.extra_headers:
                    del m.extra_headers['Cc']
                if 'Bcc' in m.extra_headers:
                    del m.extra_headers['Bcc']
                return m
            return super(_Mail, self).send_messages(
                map(process_message, email_messages))

    return _Mail



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2012, 2013, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
"""
    invenio_ext.cache
    -----------------

    This module provides initialization and configuration for `flask_cache`
    module.
"""

from flask_cache import Cache
cache = Cache()

__all__ = ['cache', 'setup_app']


def setup_app(app):
    """Setup cache extension."""

    app.config.setdefault('CACHE_TYPE',
                          app.config.get('CFG_FLASK_CACHE_TYPE', 'redis'))
    # if CACHE_KEY_PREFIX is not specified then CFG_DATABASE_NAME:: is used.
    prefix = app.config.get('CFG_DATABASE_NAME', '')
    if prefix:
        prefix += '::'
    app.config.setdefault('CACHE_KEY_PREFIX', prefix)
    cache.init_app(app)
    return app



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Jasmine test runner extension.

1. Ensure ``TESTING`` configuration variable is set to ``True``.
2. Reinstall assets (with ``TESTING = True``).

   .. code-block:: console

        (invenio)$ cdvirtualenv src/invenio-demosite
        (invenio)$ inveniomanage bower -i bower-base.json > bower.json
        (invenio)$ bower install
        (invenio)$ inveniomanage collect
        (invenio)$ inveniomanage assets build

3. Run tests by loading http://localhost:4000/jasmine/specrunner in your
   browser.

Missing features
----------------

* Console test runner (likely we will use Karma test runner).
* CI integration.


FlightJS tests
---------------

Following is an example of a very simple FlightJS component which is normally
added in your Invenio module under ``static/js/**/*.js``:

.. code-block:: javascript

    // invenio/modules/*/static/js/mycomponent.js
    define(function(require) {
        'use strict';

        var defineComponent = require('flight/lib/component');

        return defineComponent(mycomponent);

        function mycomponent() {
            this.attributes({
                myattr: 'somevalue',
            });

            // ...

            this.after('initialize', function() {
                // ...
            });
        }
    });

To test above FlightJS component, create a spec file in
``testsuite/js/**/*.spec.js``:

.. code-block:: javascript

    // invenio/modules/*/testsuite/js/mycomponent.spec.js
    'use strict';

    describeComponent('js/mycomponent', function() {
        // Initialize the component and attach it to the DOM
        beforeEach(function() {
            this.setupComponent();
        });

        it('should be defined', function() {
            expect(this.component).toBeDefined();
        });
    });


Fixtures
--------
To load fixtures (created under ``testsuite/js/**/*.html``) for the test.
Inside the spec file take following steps:

.. code-block:: javascript

   jasmine.getFixtures().fixturesPath =
       '/jasmine/spec/invenio.modules.<module_name>/';
   readFixtures('<fixture_name>')

See https://github.com/flightjs/jasmine-flight for further examples how to test
FlightJS components.

HOWTO
-----

Mock AJAX request
~~~~~~~~~~~~~~~~~

.. code-block: javascript

    'use strict';

    describeComponent('js/mycomponent', function() {

        // Initialize the component and attach it to the DOM
        beforeEach(function() {
            jasmine.Ajax.install();
            this.setupComponent();
        });

        afterEach(function() {
            jasmine.Ajax.uninstall();
        });

        it('should make ajax request', function() {
            // trigger some event that makes an AJAX request ...

            // Retrieve most recent request.
            var request = jasmine.Ajax.requests.mostRecent();
            expect(request.url).toBe('/foo/bar');
            expect(request.method).toBe('GET');

            // Fake response from server.
            request.response({
                status: 200,
                responseText: 'somevalue'
            })
        });
    });

See http://jasmine.github.io/2.0/ajax.html for further information.

"""

from __future__ import absolute_import

from .registry import specs as specs_registry
from .views import blueprint
from ..assets.registry import bundles as bundles_registry


def setup_app(app):
    """Initialize Jasmine extensions."""
    if app.testing:
        # Register blueprint
        app.register_blueprint(blueprint)

        # Register bundles
        with app.app_context():
            from . import bundles
            bundles_registry.register((bundles, bundles.jasmine_js))
            bundles_registry.register((bundles, bundles.jasmine_styles))
            # invenio/invenio/base seems to be working dir here
            specs_registry.register(
                '../ext/jasmine/testsuite/js/jasmine_configuration.spec.js',
                'invenio_ext.jasmine/jasmine_configuration.spec.js'
            )
            specs_registry.register(
                '../ext/jasmine/testsuite/js/jquery_object_mock.js',
                'invenio_ext.jasmine/jquery_object_mock.js'
            )
            specs_registry.register(
                '../ext/jasmine/testsuite/js/initialization_checker.spec.js',
                'invenio_ext.jasmine/initialization_checker.spec.js'
            )
            specs_registry.register(
                '../ext/jasmine/testsuite/js/events_checker.spec.js',
                'invenio_ext.jasmine/events_checker.spec.js'
            )
            specs_registry.register(
                '../ext/jasmine/testsuite/js/simple_div.html',
                'invenio_ext.jasmine/simple_div.html'
            )



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2012, 2013, 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Initialize and configure *Flask-Principal* extension."""

from flask import current_app
from flask_login import user_logged_in, user_logged_out
from flask_principal import Principal, Identity, AnonymousIdentity, \
    identity_changed, Permission
from six import iteritems

from .wrappers import actions, Action

__all__ = ('setup_app', 'principals', 'permission_required', 'actions',
           'Action')

principals = Principal()


class AccAuthorizeActionPermission(Permission):

    """Wrapper for ``acc_authorize_action``."""

    def __init__(self, action, **kwargs):
        """Define action and arguments."""
        self.action = action
        self.params = kwargs
        super(self.__class__, self).__init__()

    def allows(self, identity):
        """Check if given identity can perform defined action."""
        from invenio_access.engine import acc_authorize_action
        auth, message = acc_authorize_action(
            identity.id, self.action, **dict(
                (k, v() if callable(v) else v)
                for (k, v) in iteritems(self.params)))
        if auth == 0:
            return True
        current_app.logger.info(message)
        return False


def permission_required(action, **kwargs):
    """Check if user can perform given action."""
    return AccAuthorizeActionPermission(action, **kwargs).require(
        http_exception=401)


def setup_app(app):
    """Setup principal extension."""
    principals.init_app(app)

    @user_logged_in.connect_via(app)
    def _logged_in(sender, user):
        identity_changed.send(sender, identity=Identity(user.get_id()))

    @user_logged_out.connect_via(app)
    def _logged_out(sender, user):
        identity_changed.send(sender, identity=AnonymousIdentity())

    # @identity_loaded.connect_via(app)
    # def on_identity_loaded(sender, identity):
    #     """One can modify idenity object."""
    #     pass

    return app



# -*- coding: utf-8 -*-
# This file is part of Invenio.
# Copyright (C) 2012, 2013, 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""
Additional extensions and functions for the `flask_assets` module.

.. py:data:: command

    Flask-Script command that deals with assets.

    Documentation is on: `webassets` :ref:`webassets:script-commands`

    .. code-block:: python

        # How to install it
        from flask_script import Manager
        manager = Manager()
        manager.add_command("assets", command)

.. py:data:: registry

    Flask-Registry registry that handles the bundles. Use it directly as it's
    lazy loaded.
"""

from .commands import AssetsCommand, BowerCommand
from .extensions import BundleExtension
from .registry import bundles
from .wrappers import Bundle, RequireJSFilter, CleanCSSFilter


__all__ = ("bower", "bundles", "command", "setup_app", "Bundle",
           "RequireJSFilter", "CleanCSSFilter")

command = AssetsCommand()
bower = BowerCommand()


def setup_app(app):
    """Initialize Assets extension.

    Use the ``ASSETS_BUNDLES_DIR`` option to change the name of the directory
    where the assets are generated (by default ``gen``).

    :param app: Flask application
    """
    app.config.setdefault("ASSETS_BUNDLES_DIR", "gen")

    BundleExtension.install(app)

    return app



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

from flask import request, url_for
from flask_restful import Resource

from invenio_ext.restful import require_api_auth, require_header, \
    require_oauth_scopes
from invenio_ext.sqlalchemy import db
from invenio_testing import InvenioTestCase


class DecoratorsTestCase(InvenioTestCase):

    def setUp(self):
        from invenio_accounts.models import User
        from invenio_oauth2server.registry import scopes
        from invenio_oauth2server.models import Token, Scope

        # Setup variables:
        self.called = dict()

        # Setup test scopes
        with self.app.app_context():
            scopes.register(Scope(
                'test:testscope',
                group='Test',
                help_text='Test scope',
            ))

        # Setup API resources
        class Test1Resource(Resource):
            # NOTE: Method decorators are applied in reverse order
            method_decorators = [
                require_oauth_scopes('test:testscope'),
                require_api_auth(),
            ]

            def get(self):
                assert request.oauth.access_token
                return "success", 200

            def post(self):
                assert request.oauth.access_token
                return "success", 200

            @require_header('Content-Type', 'application/json')
            def put(self):
                return "success", 200

        class Test2Resource(Resource):
            @require_api_auth()
            @require_oauth_scopes('test:testscope')
            def get(self):
                assert request.oauth.access_token
                return "success", 200

            @require_api_auth()
            @require_oauth_scopes('test:testscope')
            def post(self):
                assert request.oauth.access_token
                return "success", 200

            @require_header('Content-Type', 'text/html')
            def put(self):
                return "success", 200

        # Register API resources
        api = self.app.extensions['restful']
        api.add_resource(
            Test1Resource,
            '/api/test1/decoratorstestcase/'
        )
        api.add_resource(
            Test2Resource,
            '/api/test2/decoratorstestcase/'
        )

        # Create a user
        self.user = User(
            email='info@invenio-software.org', nickname='tester'
        )
        self.user.password = "tester"
        db.session.add(self.user)
        db.session.commit()

        # Create tokens
        self.token = Token.create_personal(
            'test-', self.user.id, scopes=['test:testscope'], is_internal=True)
        self.token_noscope = Token.create_personal(
            'test-', self.user.id, scopes=[], is_internal=True)

    def tearDown(self):
        db.session.delete(self.user)
        db.session.delete(self.token.client)
        db.session.delete(self.token)
        db.session.delete(self.token_noscope.client)
        db.session.delete(self.token_noscope)
        db.session.commit()

    def test_require_api_auth_test1(self):
        res = self.client.get(url_for('test1resource'))
        self.assert401(res)
        res = self.client.get(
            url_for('test1resource', access_token=self.token.access_token))
        self.assert200(res)

    def test_require_api_auth_test2(self):
        res = self.client.get(url_for('test2resource'))
        self.assert401(res)
        res = self.client.get(
            url_for('test2resource', access_token=self.token.access_token))
        self.assert200(res)

    def test_require_oauth_scopes_test1(self):
        res = self.client.post(
            url_for('test1resource', access_token=self.token.access_token))
        self.assert200(res)
        res = self.client.post(
            url_for('test1resource',
                    access_token=self.token_noscope.access_token))
        self.assertStatus(res, 403)

    def test_require_oauth_scopes_test2(self):
        res = self.client.post(
            url_for('test2resource', access_token=self.token.access_token))
        self.assert200(res)
        res = self.client.post(
            url_for('test2resource',
                    access_token=self.token_noscope.access_token))
        self.assertStatus(res, 403)

    def test_require_header_test1(self):
        res = self.client.put(
            url_for('test1resource', access_token=self.token.access_token),
            headers=[('Content-Type', 'application/json')])
        self.assert200(res)
        res = self.client.put(
            url_for('test1resource', access_token=self.token.access_token),
            headers=[('Content-Type', 'text/html')])
        self.assertStatus(res, 415)

    def test_require_header_test2(self):
        res = self.client.put(
            url_for('test2resource'),
            headers=[('Content-Type', 'text/html; charset=UTF-8')])
        self.assert200(res)
        res = self.client.put(
            url_for('test2resource'),
            headers=[('Content-Type', 'application/json')])
        self.assertStatus(res, 415)


class RestfulPaginationTestCase(InvenioTestCase):

    def setUp(self):
        """Set up some dummy data and a resource."""
        from invenio_accounts.models import User
        from invenio_oauth2server.models import Token

        self.data = range(25)

        # setup test api resources

        class TestDataResource(Resource):

            method_decorators = [
                require_api_auth()
            ]

            @require_header('Content-Type', 'application/json')
            def get(self):
                import json
                from flask import make_response
                from invenio_ext.restful.errors import(
                    InvalidPageError
                )
                from invenio_ext.restful import pagination
                # Test to see that the exceptions are raised correctly
                # In restful.py it is not needed because the error_hanler
                # takes care of exceptions
                response = None
                try:
                    # test data
                    testdata = range(25)
                    endpoint = request.endpoint
                    args = request.args
                    page = int(args.get('page', 1))
                    per_page = int(args.get('per_page', 10))
                    p = pagination.RestfulPagination(
                        page=page, per_page=per_page, total_count=len(testdata)
                    )
                    data_to_return = p.slice(testdata)
                    kwargs = {}
                    kwargs['endpoint'] = endpoint
                    kwargs['args'] = request.args
                    link_header = p.link_header(**kwargs)
                    response = make_response(json.dumps(data_to_return))
                    response.headers[link_header[0]] = link_header[1]
                    response.headers[
                        'Content-Type'] = request.headers['Content-Type']
                except InvalidPageError as e:
                    exception = {}
                    exception['message'] = e.error_msg
                    exception['type'] = "{0}".format(type(e))
                    response = make_response(json.dumps(exception))
                return response

        # Register API resources
        api = self.app.extensions['restful']
        api.add_resource(
            TestDataResource,
            '/api/testdata/'
        )

        # Create a user
        self.user = User(
            email='info@invenio-software.org', nickname='tester'
        )
        self.user.password = "tester"
        db.session.add(self.user)
        db.session.commit()

        # create token
        self.token = Token.create_personal(
            'test-', self.user.id, scopes=[], is_internal=True)

    def tearDown(self):
        """Delete the dummy data."""
        del self.data
        db.session.delete(self.user)
        db.session.delete(self.token.client)
        db.session.delete(self.token)
        db.session.commit()

    def test_paginate_page1(self):
        endpoint = "/api/testdata/?"
        link_template = '<{}per_page={}&page={}>; rel="{}"'
        answer_get = self.client.get(
            url_for('testdataresource', access_token=self.token.access_token,
                    per_page=10),
            headers=[('Content-Type', 'application/json')])
        data_returned = answer_get.json
        links_string = answer_get.headers['Link']
        # expected answers
        expected_data = self.data[0:10]
        first_link = link_template.format(endpoint, 10, 1, "first")
        next_link = link_template.format(endpoint, 10, 2, "next")
        last_link = link_template.format(endpoint, 10, 3, "last")
        expected_links_string = "{0},{1},{2}".format(
            first_link,
            next_link,
            last_link
        )
        self.assertEqual(data_returned, expected_data)
        self.assertEqual(links_string, expected_links_string)

    def test_paginate_page2(self):
        endpoint = "/api/testdata/?"
        link_template = '<{}per_page={}&page={}>; rel="{}"'
        answer_get = self.client.get(
            url_for('testdataresource', access_token=self.token.access_token,
                    page=2, per_page=10),
            headers=[('Content-Type', 'application/json')])
        data_returned = answer_get.json
        links_string = answer_get.headers['Link']
        # expected answers
        expected_data = self.data[10:20]
        first_link = link_template.format(endpoint, 10, 1, "first")
        prev_link = link_template.format(endpoint, 10, 1, "prev")
        next_link = link_template.format(endpoint, 10, 3, "next")
        last_link = link_template.format(endpoint, 10, 3, "last")
        expected_links_string = "{0},{1},{2},{3}".format(
            first_link,
            prev_link,
            next_link,
            last_link
        )
        self.assertEqual(data_returned, expected_data)
        self.assertEqual(links_string, expected_links_string)

    def test_paginate_lastpage(self):
        endpoint = "/api/testdata/?"
        link_template = '<{}per_page={}&page={}>; rel="{}"'
        answer_get = self.client.get(
            url_for('testdataresource', access_token=self.token.access_token,
                    page=3, per_page=10),
            headers=[('Content-Type', 'application/json')])
        data_returned = answer_get.json
        links_string = answer_get.headers['Link']

        # expected answers
        expected_data = self.data[20:25]
        first_link = link_template.format(endpoint, 10, 1, "first")
        prev_link = link_template.format(endpoint, 10, 2, "prev")
        last_link = link_template.format(endpoint, 10, 3, "last")
        expected_links_string = "{0},{1},{2}".format(
            first_link,
            prev_link,
            last_link
        )
        self.assertEqual(data_returned, expected_data)
        self.assertEqual(links_string, expected_links_string)

    def test_paginate_nonexistentpage(self):
        from invenio_ext.restful import errors
        answer_get = self.client.get(
            url_for('testdataresource',
                    access_token=self.token.access_token,
                    page=-2, per_page=10),
            headers=[('Content-Type', 'application/json')])
        # Test/assert to see that the exceptions are raised correctly
        expected = {}
        error_msg = "Invalid page number ('{0}').".format(-2)
        expected['message'] = error_msg
        expected['type'] = "{0}".format(errors.InvalidPageError)
        self.assertEqual(answer_get.json, expected)

    def test_paginate_per_pageerror(self):
        from invenio_ext.restful import errors
        answer_get = self.client.get(
            url_for('testdataresource',
                    access_token=self.token.access_token,
                    page=1, per_page=-5),
            headers=[('Content-Type', 'application/json')])
        # Test/assert to see that the exceptions are raised correctly
        expected = {}
        error_msg = (
            "Invalid per_page argument ('{0}'). Number of items "
            "per pages must be positive integer.".format(-5)
        )
        expected['message'] = error_msg
        expected['type'] = "{0}".format(errors.InvalidPageError)
        self.assertEqual(answer_get.json, expected)


class WtgTAGPaginationMokup:

    total = 6

    class TagMockup():
        def __init__(self, id):
            self.id = id
            self.name = 'tag' + str(id)
            self.id_user = id + 1

    @staticmethod
    def items():
        items = list()
        for i in range(WtgTAGPaginationMokup.total):
            obj = WtgTAGPaginationMokup.TagMockup(id=i)
            items.append(obj)
        return items

    class paginate:

        def __init__(self, page, per_page, error_out=True):
            self.page = page
            self.per_page = per_page
            self.pages = int(WtgTAGPaginationMokup.total / per_page) + 1

        @property
        def has_next(self):
            return (self.page * self.per_page) < WtgTAGPaginationMokup.total

        @property
        def has_prev(self):
            return ((self.page - 1) * self.per_page) > 0

        @property
        def items(self):
            return WtgTAGPaginationMokup.items()[
                (self.page - 1) * self.per_page:self.page * self.per_page]


class RestfulSQLAlchemyPaginationTestCase(InvenioTestCase):

    def setUp(self):
        from flask_restful import Resource, fields, marshal
        from invenio_accounts.models import User
        from invenio_oauth2server.models import Token

        class TagRepresenation(object):

            """A representation of a tag.

            This class will be only used to return a tag as JSON.
            """

            marshaling_fields = dict(
                id=fields.Integer,
                name=fields.String,
                id_user=fields.Integer
            )

            def __init__(self, retrieved_tag):
                """Initialization.

                Declared the attributes to marshal with a tag.
                :param retrieved_tag: a tag from the database
                """
                # get fields from the given tag
                self.id = retrieved_tag.id
                self.name = retrieved_tag.name
                self.id_user = retrieved_tag.id_user

            def marshal(self):
                """Marshal the Tag."""
                return marshal(self, self.marshaling_fields)

        class TestTagsResource(Resource):

            method_decorators = [
                require_api_auth()
            ]

            @require_header('Content-Type', 'application/json')
            def get(self):
                import json
                from flask import make_response
                from invenio_ext.restful.errors import(
                    RestfulError, InvalidPageError
                )
                from invenio_ext.restful import pagination

                response = None
                try:
                    endpoint = request.endpoint
                    args = request.args
                    page = int(args.get('page', 1))
                    per_page = int(args.get('per_page', 2))
                    # check values arguments and raise exceptions if any errors
                    if per_page < 0:
                        raise RestfulError(
                            error_msg="Invalid per_page: {}".format(per_page),
                            status_code=400
                        )
                    if page < 0:
                        raise InvalidPageError(
                            error_msg="Invalid page: {}".format(page),
                            status_code=400
                        )

                    # need to sort by id
                    # also assuming only one user so no need to filter
                    # user's id
                    tags_q = WtgTAGPaginationMokup()
                    p = pagination.RestfulSQLAlchemyPagination(
                        query=tags_q, page=page, per_page=per_page
                    )
                    if page > p.pages:
                        raise InvalidPageError(
                            error_msg="Invalid page: {}".format(page),
                            status_code=400
                        )
                    tags_to_return = map(
                        lambda x: TagRepresenation(x).marshal(),
                        p.items
                    )

                    kwargs = {}
                    kwargs['endpoint'] = endpoint
                    kwargs['args'] = request.args
                    link_header = p.link_header(**kwargs)
                    response = make_response(json.dumps(tags_to_return))
                    response.headers[link_header[0]] = link_header[1]
                    response.headers[
                        'Content-Type'] = request.headers['Content-Type']
                except (RestfulError, InvalidPageError) as e:
                    exception = {}
                    exception['message'] = e.error_msg
                    exception['type'] = "{0}".format(type(e))
                    response = make_response(json.dumps(exception))
                return response

        # Register API resources
        api = self.app.extensions['restful']
        api.add_resource(
            TestTagsResource,
            '/api/testtags/'
        )

        # Create a user
        self.user = User(
            email='info@invenio-software.org', nickname='tester'
        )
        self.user.password = "tester"
        db.session.add(self.user)
        db.session.commit()

        # create token
        self.token = Token.create_personal(
            'test-', self.user.id, scopes=[], is_internal=True)

    def tearDown(self):
        db.session.delete(self.user)
        db.session.delete(self.token.client)
        db.session.delete(self.token)
        db.session.commit()

    def test_pagination_flow(self):
        # endpoint
        endpoint = "/api/testtags/?"
        # links template
        link_template = '<{}per_page={}&page={}>; rel="{}"'

        # request first page
        answer_get = self.client.get(
            url_for('testtagsresource', access_token=self.token.access_token,
                    page=1, per_page=2),
            headers=[('Content-Type', 'application/json')])
        # check to ensure correct results
        tags_names_from_request = [x['name'] for x in answer_get.json]
        links_string = answer_get.headers['Link']
        expected_names = []
        for i in range(0, 2):
            expected_name = WtgTAGPaginationMokup.items()[i].name
            expected_names.append(expected_name)

        first_link = link_template.format(endpoint, 2, 1, "first")
        next_link = link_template.format(endpoint, 2, 2, "next")
        last_link = link_template.format(endpoint, 2, 4, "last")
        expected_links_string = "{0},{1},{2}".format(
            first_link,
            next_link,
            last_link
        )
        self.assertEqual(set(tags_names_from_request), set(expected_names))
        self.assertEqual(links_string, expected_links_string)

        tags_names_from_request = []
        expected_names = []
        # request second page
        answer_get = self.client.get(
            url_for('testtagsresource', access_token=self.token.access_token,
                    page=2, per_page=2),
            headers=[('Content-Type', 'application/json')])
        # check to ensure correct results
        tags_names_from_request = [x['name'] for x in answer_get.json]
        links_string = answer_get.headers['Link']
        # check if names of tags are the expected ones
        expected_names = []
        for i in range(2, 4):
            expected_name = WtgTAGPaginationMokup.items()[i].name
            expected_names.append(expected_name)

        first_link = link_template.format(endpoint, 2, 1, "first")
        prev_link = link_template.format(endpoint, 2, 1, "prev")
        next_link = link_template.format(endpoint, 2, 3, "next")
        last_link = link_template.format(endpoint, 2, 4, "last")
        expected_links_string = "{0},{1},{2},{3}".format(
            first_link,
            prev_link,
            next_link,
            last_link
        )

        self.assertEqual(set(tags_names_from_request), set(expected_names))
        self.assertEqual(links_string, expected_links_string)

        tags_names_from_request = []
        expected_names = []
        # request third(last) page
        answer_get = self.client.get(
            url_for('testtagsresource', access_token=self.token.access_token,
                    page=3, per_page=2),
            headers=[('Content-Type', 'application/json')])
        # check to ensure correct results
        tags_names_from_request = [x['name'] for x in answer_get.json]
        links_string = answer_get.headers['Link']
        # check if names of tags are the expected ones
        expected_names = []
        for i in range(4, 6):
            expected_name = WtgTAGPaginationMokup.items()[i].name
            expected_names.append(expected_name)

        first_link = link_template.format(endpoint, 2, 1, "first")
        prev_link = link_template.format(endpoint, 2, 2, "prev")
        last_link = link_template.format(endpoint, 2, 4, "last")
        expected_links_string = "{0},{1},{2}".format(
            first_link,
            prev_link,
            last_link
        )

        self.assertEqual(set(tags_names_from_request), set(expected_names))
        self.assertEqual(links_string, expected_links_string)

    def test_paginate_nonexistentpage(self):
        from invenio_ext.restful import errors
        answer_get = self.client.get(
            url_for('testtagsresource',
                    access_token=self.token.access_token,
                    page=-2),
            headers=[('Content-Type', 'application/json')])
        # Test/assert to see that the exceptions are raised correctly
        expected = {}
        expected['message'] = "Invalid page: {0}".format(-2)
        expected['type'] = "{0}".format(errors.InvalidPageError)
        self.assertEqual(answer_get.json, expected)

    def test_paginate_per_pageerror(self):
        from invenio_ext.restful import errors
        answer_get = self.client.get(
            url_for('testtagsresource',
                    access_token=self.token.access_token,
                    per_page=-5),
            headers=[('Content-Type', 'application/json')])
        # Test/assert to see that the exceptions are raised correctly
        expected = {}
        expected['message'] = "Invalid per_page: {0}".format(-5)
        expected['type'] = "{0}".format(errors.RestfulError)
        self.assertEqual(answer_get.json, expected)



# -*- coding: utf-8 -*-
# This file is part of Invenio.
# Copyright (C) 2011, 2012, 2013, 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Tools for working with legacy application."""

from __future__ import print_function

import os

import sys

from flask import abort, current_app, render_template, send_from_directory

from invenio_base import signals
from invenio_base.scripts.database import create, recreate
from invenio_base.utils import run_py_func

from .request_class import LegacyRequest


def cli_cmd_reset(sender, yes_i_know=False, drop=True, **kwargs):
    """Reset legacy values."""
    cmds = []
    if 'invenio_access' in current_app.config['PACKAGES']:
        from invenio_access.scripts.webaccessadmin import main as \
            webaccessadmin
        cmds.append(
            (webaccessadmin, "webaccessadmin -u admin -c -a -D")
        )

    for cmd in cmds:
        if run_py_func(*cmd, passthrough=True).exit_code:
            print("ERROR: failed execution of", *cmd)
            sys.exit(1)

signals.post_command.connect(cli_cmd_reset, sender=create)
signals.post_command.connect(cli_cmd_reset, sender=recreate)


def setup_app(app):
    """Setup up the app."""
    # Legacy config support
    _use_x_sendfile = app.config.get('CFG_BIBDOCFILE_USE_XSENDFILE')
    _debug = app.config.get('CFG_DEVEL_SITE', 0) > 0
    app.config.setdefault('USE_X_SENDFILE', _use_x_sendfile)
    app.config.setdefault('DEBUG', _debug)
    app.debug = app.config['DEBUG']

    # Legacy directory that must exist
    for cfg_dir in ['CFG_BATCHUPLOADER_DAEMON_DIR',
                    'CFG_BIBDOCFILE_FILEDIR',
                    'CFG_BIBENCODE_DAEMON_DIR_NEWJOBS',
                    'CFG_BIBENCODE_DAEMON_DIR_OLDJOBS',
                    'CFG_BIBENCODE_TARGET_DIRECTORY',
                    'CFG_BINDIR',
                    'CFG_CACHEDIR',
                    'CFG_ETCDIR',
                    'CFG_LOCALEDIR',
                    'CFG_LOGDIR',
                    'CFG_PYLIBDIR',
                    'CFG_RUNDIR',
                    'CFG_TMPDIR',
                    'CFG_TMPSHAREDDIR',
                    'CFG_WEBBASKET_DIRECTORY_BOX_NUMBER_OF_COLUMNS',
                    'CFG_WEBDIR',
                    'CFG_WEBSUBMIT_BIBCONVERTCONFIGDIR',
                    'CFG_WEBSUBMIT_COUNTERSDIR',
                    'CFG_WEBSUBMIT_STORAGEDIR']:
        path = app.config.get(cfg_dir)
        if path:
            try:
                if not os.path.exists(path):
                    os.makedirs(path)
            except OSError:
                app.logger.exception("Cannot property create directory {path} "
                                     "for legacy variable {cfg_dir}"
                                     .format(path=path, cfg_dir=cfg_dir))

    # Set custom request class.
    app.request_class = LegacyRequest

    @app.errorhandler(404)
    @app.errorhandler(405)
    def page_not_found(error):
        if error.code == 404:
            return render_template('404.html'), 404
        return str(error), error.code

    @app.route('/admin/<module>/<action>.py', methods=['GET', 'POST', 'PUT'])
    @app.route('/admin/<module>/<action>.py/<path:arguments>',
               methods=['GET', 'POST', 'PUT'])
    def web_admin(module, action, arguments=None):
        """Add support for legacy mod publisher."""
        return render_template('404.html'), 404

    @app.endpoint('static')
    def static_handler_with_legacy_publisher(*args, **kwargs):
        """Serve static files from instance path."""
        # Static file serving for devserver
        # ---------------------------------
        # Apache normally serve all static files, but if we are using the
        # devserver we need to serve static files here.
        filename = kwargs.get("filename")
        if not app.config.get('CFG_FLASK_SERVE_STATIC_FILES') \
                or filename is None or app.static_folder is None:
            abort(404)
        else:
            return send_from_directory(app.static_folder, filename)

    return app



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Registry definition for fixture datasets."""

from flask_registry import RegistryProxy

from invenio_ext.registry import ModuleAutoDiscoveryRegistry
from invenio_utils.datastructures import LazyDict

fixtures_proxy = RegistryProxy(
    'fixtures', ModuleAutoDiscoveryRegistry, 'fixtures')


def fixtures_loader():
    """Load fixtures datasets."""
    out = {}
    for fixture in fixtures_proxy:
        for data in getattr(fixture, '__all__', dir(fixture)):
            if data[-4:] != 'Data' or data in out:
                continue
            out[data] = getattr(fixture, data)
    return out

fixtures = LazyDict(fixtures_loader)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2011, 2012, 2013, 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Initialize session interface and default config variables."""

from .interface import SessionInterface

ONE_DAY = 86400


def setup_app(app):
    """Create custom session interface."""
    app.config.setdefault('SESSION_COOKIE_NAME', 'INVENIOSESSION')
    app.config.setdefault(
        'PERMANENT_SESSION_LIFETIME',
        app.config.get('CFG_WEBSESSION_EXPIRY_LIMIT_REMEMBER') * ONE_DAY)
    # Attach legacy session handling.
    app.session_interface = SessionInterface()
    return app



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2012, 2013, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

# Copyright (c) 2012, Kenneth Reitz
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
# Redistributions of source code must retain the above copyright notice, this list
# of conditions and the following disclaimer.
# Redistributions in binary form must reproduce the above copyright notice, this
# list of conditions and the following disclaimer in the documentation and/or
# other materials provided with the distribution.
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
# ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


from flask import redirect, request

YEAR_IN_SECS = 31536000


class SSLify(object):
    """Secures your Flask App."""

    def __init__(self, app, age=YEAR_IN_SECS, subdomains=False, permanent=False,
                 doHSTS=True):
        if app is not None:
            self.app = app
            self.hsts_age = age
            self.hsts_include_subdomains = subdomains
            self.permanent = permanent
            self.criteria_callback = None

            self.init_app(self.app, doHSTS)
        else:
            self.app = None

    def init_app(self, app, doHSTS=True):
        """Configures the configured Flask app to enforce SSL."""
        app.before_request(self.redirect_to_ssl)
        if doHSTS:
            app.after_request(self.set_hsts_header)

    def criteria_handler(self, callback):
        """Sets criteria callback."""
        self.criteria_callback = callback

    @property
    def hsts_header(self):
        """Returns the proper HSTS policy."""
        hsts_policy = 'max-age={0}'.format(self.hsts_age)

        if self.hsts_include_subdomains:
            hsts_policy += '; includeSubDomains'

        return hsts_policy

    def redirect_to_ssl(self):
        """Redirect incoming requests to HTTPS."""
        # Should we redirect?
        criteria = [
            request.is_secure,
            request.headers.get('X-Forwarded-Proto', 'http') == 'https',
            'no-https-redirect' in self.app.config.get('CFG_DEVEL_TOOLS', []),
        ]

        if self.criteria_callback is not None:
            criteria += [self.criteria_callback()]

        if not any(criteria):
            if request.url.startswith('http://'):
                from invenio_utils.url import rewrite_to_secure_url
                url = rewrite_to_secure_url(request.url)

                code = 302
                if self.permanent:
                    code = 301
                r = redirect(url, code=code)

                return r

    def set_hsts_header(self, response):
        """Adds HSTS header to each response."""
        if request.is_secure:
            response.headers.setdefault(
                'Strict-Transport-Security', self.hsts_header)
        return response



# -*- coding: utf-8 -*-
# This file is part of Invenio.
# Copyright (C) 2012, 2013, 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""This module fixes problems with missing ``invenio.config`` module."""

import os
import sys
import warnings

from six import iteritems

from invenio_utils.deprecation import RemovedInInvenio23Warning


def get_relative_url(url):
    """Return the relative URL from a URL.

     For example:::

        'http://web.net' -> ''
        'http://web.net/' -> ''
        'http://web.net/1222' -> '/1222'
        'http://web.net/wsadas/asd' -> '/wsadas/asd'

    It will never return a trailing "/".

    :param url: A url to transform
    :type url: str

    :return: relative URL
    """
    # remove any protocol info before
    stripped_site_url = url.replace("://", "")
    baseurl = "/" + "/".join(stripped_site_url.split("/")[1:])

    # remove any trailing slash ("/")
    if baseurl[-1] == "/":
        return baseurl[:-1]
    else:
        return baseurl


def setup_app(app):
    """The extension wraps application configs.

    This extension wraps application configs and installs it as python module
    to ``invenio.config``.
    As a next step it sets an attribute ``config`` to ``invenio`` module
    to enable import statment `from invenio import config`.
    """
    # STEP 0: Special treatment of base URL, adding CFG_BASE_URL.
    CFG_BASE_URL = get_relative_url(app.config.get("CFG_SITE_URL"))
    app.config['CFG_BASE_URL'] = CFG_BASE_URL

    # STEP 1: create simple :class:`.Wrapper`.
    class Wrapper(object):

        """Wrapper."""

        def __init__(self, wrapped):
            """Init."""
            self.wrapped = wrapped

        def __getattr__(self, name):
            """Get attr."""
            # Perform custom logic here
            if name == '__file__':
                return __file__
            elif name == '__path__':
                return os.path.dirname(__file__)
            try:
                from invenio_base.helpers import utf8ifier
                warnings.warn(
                    "Usage of invenio.config.{0} is deprecated".format(name),
                    RemovedInInvenio23Warning
                )
                return utf8ifier(self.wrapped[name])
            except Exception:
                pass
                # import traceback
                # traceback.print_stack()

    # STEP 2: wrap application config and sets it as `invenio.config` module.
    sys.modules['invenio.config'] = Wrapper(app.config)
    sys.modules['invenio.dbquery_config'] = Wrapper(dict(
        (k, v) for (k, v) in iteritems(app.config)
        if k.startswith('CFG_DATABASE')))

    return app



# -*- coding: utf-8 -*-
# This file is part of Invenio.
# Copyright (C) 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Custom modified classes."""

from __future__ import absolute_import, print_function, unicode_literals

import os

from flask import current_app
from flask_assets import Bundle as BundleBase
from flask_registry import ModuleAutoDiscoveryRegistry
from webassets.filter import ExternalTool
from webassets.filter.requirejs import RequireJSFilter as RequireJSFilterBase
from werkzeug.utils import import_string


class Bundle(BundleBase):

    """
    Bundle extension with a name and bower dependencies.

    The name is only used for the requirements from the templates and the
    weight does the bundle ordering.

    The bower dependencies are used to generate a bower.json file.
    """

    def __init__(self, *contents, **options):
        """
        Initialize the named bundle.

        :param name: name of the bundle
        :type name: str
        :param bower: bower dependencies
        :type bower: dict
        :param weight: weight of the bundle, lighter are loaded first.
        :type weight: int
        """
        self.bower = options.pop("bower", {})
        self.weight = int(options.pop("weight", 50))
        super(Bundle, self).__init__(*contents, **options)

        # ease the bundle modification
        self.contents = list(self.contents)
        self.app = options.pop("app", None)

    def has_filter(self, *filters):
        """Tell whether a given filter is set up for this bundle."""
        for f in self.filters:
            if f.name in filters:
                return True
        return False


class BundlesAutoDiscoveryRegistry(ModuleAutoDiscoveryRegistry):

    """
    Registry that searches for bundles.

    Its registry is a list of the package name and the bundle itself. This way
    you can keep track of where a bundle was loaded from.
    """

    def __init__(self, module_name=None, app=None, with_setup=False,
                 silent=False):
        """
        Initialize the bundle auto discovery registry.

        :param module_name: where to look for bundles (default: bundles)
        :type module_name: str

        """
        super(BundlesAutoDiscoveryRegistry, self).__init__(
            module_name or 'bundles', app=app, with_setup=with_setup,
            silent=silent)

    def _discover_module(self, module):
        """Discover the bundles in the given module."""
        import_str = module + '.' + self.module_name

        # FIXME this boilerplate code should be factored out in Flask-Registry.
        try:
            bundles = import_string(import_str, silent=self.silent)
        except ImportError as e:
            self._handle_importerror(e, module, import_str)
        except SyntaxError as e:
            self._handle_syntaxerror(e, module, import_str)
        else:
            variables = getattr(bundles, "__all__", dir(bundles))
            for var in variables:
                # ignore private/protected fields
                if var.startswith('_'):
                    continue
                bundle = getattr(bundles, var)
                if isinstance(bundle, Bundle):
                    self.register((module, bundle))


class RequireJSFilter(RequireJSFilterBase):

    """Optimize AMD-style modularized JavaScript into a single asset.

    Adds support for exclusion of files already in defined in other bundles.
    """

    def __init__(self, *args, **kwargs):
        """Initialize filter."""
        self.excluded_bundles = kwargs.pop('exclude', [])
        super(RequireJSFilter, self).__init__(*args, **kwargs)

    def setup(self):
        """Setup filter (only called when filter is actually used)."""
        super(RequireJSFilter, self).setup()

        excluded_files = []
        for bundle in self.excluded_bundles:
            excluded_files.extend(
                map(lambda f: os.path.splitext(f)[0],
                    bundle.contents)
            )

        if excluded_files:
            self.argv.append(
                "exclude={0}".format(",".join(excluded_files))
            )


class CleanCSSFilter(ExternalTool):

    """Minify css using cleancss.

    Implements opener capable of rebasing relative CSS URLs against
    ``COLLECT_STATIC_ROOT``.
    """

    name = 'cleancssurl'
    method = 'open'
    options = {
        'binary': 'CLEANCSS_BIN',
    }

    def setup(self):
        """Initialize filter just before it will be used."""
        super(CleanCSSFilter, self).setup()
        self.root = current_app.config.get('COLLECT_STATIC_ROOT')

    def open(self, out, source_path, **kw):
        """Open source."""
        self.subprocess(
            [self.binary or 'cleancss', '--root', self.root, source_path],
            out
        )

    def output(self, _in, out, **kw):
        """Output filtering."""
        self.subprocess([self.binary or 'cleancss'], out, _in)

    def input(self, _in, out, **kw):
        """Input filtering."""
        self.subprocess([self.binary or 'cleancss'], out, _in)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2012, 2013, 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Provide initialization and configuration for `flask_login` module."""

import urllib
from datetime import datetime

from flask import current_app, flash, g, redirect, request, session, url_for

from flask_login import LoginManager, current_user, \
    login_user as flask_login_user, logout_user, user_logged_in

from invenio_base.wrappers import lazy_import

UserInfo = lazy_import('invenio_ext.login.legacy_user:UserInfo')


def login_user(user, *args, **kwargs):
    """Allow login user by its id."""
    if type(user) in [int, long]:
        user = UserInfo(user)
    return flask_login_user(user, *args, **kwargs)


def login_redirect(referer=None):
    """Redirect to url after login."""
    if referer is None:
        referer = request.values.get('referer')
    if referer:
        from six.moves.urllib.parse import urlparse
        # we should not redirect to these URLs after login
        blacklist = [url_for('webaccount.register'),
                     url_for('webaccount.logout'),
                     url_for('webaccount.login'),
                     url_for('webaccount.lost')]
        if not urlparse(referer).path in blacklist:
            # Change HTTP method to https if needed.
            from invenio_utils.url import rewrite_to_secure_url
            referer = rewrite_to_secure_url(referer)
            return redirect(referer)
    return redirect('/')


def authenticate(nickname_or_email=None, password=None,
                 login_method=None, remember=False):
    """
    Find user identified by given information and login method.

    :param nickname_or_email: User nickname or email address
    :param password: Password used only in login methods that need it
    :param login_method: Login method (default: 'Local')
    :return: UserInfo
    """
    from invenio_base.i18n import _
    from invenio_ext.sqlalchemy import db
    from invenio_accounts.models import User
    from sqlalchemy.orm.exc import NoResultFound

    where = [db.or_(User.nickname == nickname_or_email,
                    User.email == nickname_or_email)]

    try:
        user = User.query.filter(*where).one()
        if login_method == 'Local' and password is not None:
            if not user.verify_password(password, migrate=True):
                return False
    except NoResultFound:
        return None
    except Exception:
        current_app.logger.exception("Problem checking password.")
        return False

    if login_method is not None and \
            user.settings['login_method'] != login_method:
        flash(_("You are not authorized to use '%(x_login_method)s' login "
                "method.", x_login_method=login_method), 'error')
        return False

    if user.note == '2':  # account is not confirmed
        flash(_("You have not yet verified your email address."), 'warning')

    if user.note == '0':  # account is blocked
        logout_user()
        return False

    if remember:
        session.permanent = True
    return login_user(user.id, remember=remember)


def setup_app(app):
    """Setup login extension."""
    app.config.setdefault('CFG_OPENID_AUTHENTICATION', False)
    app.config.setdefault('CFG_OAUTH1_AUTHENTICATION', False)
    app.config.setdefault('CFG_OAUTH2_AUTHENTICATION', False)

    @app.errorhandler(401)
    def do_login_first(error=401):
        """Display login page when user is not authorised."""
        if request.is_xhr:
            return g._("Authorization failure"), 401
        secure_url = url_for(request.endpoint, _external=True, _scheme='https',
                             **request.view_args)
        if not urllib.unquote(secure_url).startswith(request.base_url):
            return redirect(secure_url)
        if current_user.is_guest:
            if not session.get('_flashes'):
                flash(g._("Please sign in to continue."), 'info')
            from invenio_accounts.views.accounts import login
            return login(referer=request.url)
        else:
            from flask import render_template
            return render_template("401.html"), 401

    # Let's create login manager.
    _login_manager = LoginManager(app)
    _login_manager.login_view = app.config.get('CFG_LOGIN_VIEW',
                                               'webaccount.login')
    _login_manager.anonymous_user = UserInfo
    _login_manager.unauthorized_handler(do_login_first)

    @user_logged_in.connect_via(app)
    def _logged_in(sender, user):
        """Update last login date."""
        from invenio_accounts.models import User
        User.query.filter_by(id=user.get_id()).update(dict(
            last_login=datetime.now()
        ))

    @_login_manager.user_loader
    def _load_user(uid):
        """Do not raise an exception if uid is not valid or missing."""
        return UserInfo(int(uid))

    return app



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Simplified Elastic Search integration."""

from __future__ import absolute_import

from elasticsearch import Elasticsearch
from elasticsearch.connection import RequestsHttpConnection

from invenio_celery import celery

es = None

SEARCH_RECORD_MAPPING = {
    "settings": {
        "analysis": {
            "filter": {
                "asciifold_with_orig": {
                    "type": "asciifolding",
                    "preserve_original": True
                },

                "synonyms_kbr": {
                    "type": "synonym",
                    "synonyms": [
                        "production => creation"
                    ]
                }
            },
            "analyzer": {
                "natural_text": {
                    "type": "custom",
                    "tokenizer": "standard",
                    "filter": [
                        "asciifold_with_orig",
                        "lowercase",
                        "synonyms_kbr"
                    ]
                },
                "basic_analyzer": {
                    "type": "custom",
                    "tokenizer": "standard",
                    "filter": [
                        "asciifold_with_orig",
                        "lowercase"
                    ]
                }
            }
        },
        "index.percolator.map_unmapped_fields_as_string": True,
    },
    "mappings": {
        "record": {
            "_all": {"enabled": False},
            "date_detection": False,
            "numeric_detection": False,
            "dynamic_templates": [
                {"default": {
                    "match_mapping_type": "string",
                    "mapping": {
                        "analyzer": "basic_analyzer",
                        "type": "string",
                        "copy_to": "global_default"
                    }
                }
                }
            ],
            "properties": {
                "global_fulltext": {
                    "type": "string",
                    "analyzer": "natural_text"
                },
                "global_default": {
                    "type": "string",
                    "analyzer": "basic_analyzer"
                },
                "_collections": {
                    "type": "string",
                    "index": "not_analyzed"
                },
                "collections": {
                    "properties": {
                        "primary": {
                            "type": "string",
                            "index": "not_analyzed"
                        },
                        "secondary": {
                            "type": "string",
                            "index": "not_analyzed"
                        }
                    }
                },
                "authors": {
                    "type": "string",
                    "fields": {
                        "raw": {
                            "type": "string",
                            "index": "not_analyzed"
                        }
                    }
                },
                "main_entry_personal_name": {
                    "type": "object",
                    "properties": {
                        "personal_name": {
                            "type": "string",
                            "copy_to": ["authors"],
                            "analyzer": "natural_text"
                        }
                    }
                },
                "added_entry_personal_name": {
                    "type": "object",
                    "properties": {
                        "personal_name": {
                            "type": "string",
                            "copy_to": ["authors"],
                            "analyzer": "natural_text"
                        }
                    }
                },
                "abstract": {
                    "type": "string",
                    "analyzer": "natural_text"
                },
                "title": {
                    "type": "string",
                    "analyzer": "natural_text"
                },
                "title_statement": {
                    "type": "object",
                    "properties": {
                        "title": {
                            "type": "string", "copy_to": ["title", "global_fulltext"],
                            "analyzer": "natural_text"
                        }
                    }
                },
                "division": {
                    "type": "string"
                },
                "experiment": {
                    "type": "string"
                },
                "varying_form_of_title": {
                    "type": "object",
                    "properties": {
                        "title_proper_short_title": {
                            "type": "string", "copy_to": ["title", "global_fulltext"],
                            "analyzer": "natural_text"
                        }
                    }
                },
                "summary_": {
                    "type": "object",
                    "properties": {
                        "summary_": {
                            "type": "string", "copy_to": ["abstract", "global_fulltext"],
                            "analyzer": "natural_text"
                        }
                    }
                },
                "date_and_time_of_latest_transaction": {
                    "type": "date",
                    "format": "yyyy||yyyyMM||yyyyMMdd||yyyyMMddHHmmss||yyyyMMddHHmmss.S",
                },
                "publication_date": {
                    "type": "date",
                    "format": "yyyy||yyyyMM||yyyyMMdd||yyyyMMddHHmmss||yyyyMMddHHmmss.S||dd MM yyyy||dd MMM yyyy||MMM yyyy||MMM yyyy?||yyyy ('repr'.1964.)",
                },
                "publication_distribution_imprint": {
                    "type": "object",
                    "properties": {
                        "date_of_publication_distribution": {
                            "type": "date",
                            "format": "yyyy||yyyyMM||yyyyMMdd||yyyyMMddHHmmss||yyyyMMddHHmmss.S||dd MM yyyy||dd MMM yyyy||MMM yyyy||MMM yyyy?||yyyy ('repr'.1964.)",
                            "copy_to": ["publication_date"]
                        },
                        "name_of_publisher_distributor": {
                            "type": "string",
                            "analyzer": "basic_analyzer"

                        },
                        "place_of_publication_distribution": {
                            "type": "string",
                            "analyzer": "basic_analyzer"
                        }
                    }
                }
            }
        }
    }
}


def create_index(sender, **kwargs):
    """Create or recreate the elasticsearch index for records."""
    es.indices.delete(index='records', ignore=404)
    es.indices.create(index='records', body=SEARCH_RECORD_MAPPING)


def delete_index(sender, **kwargs):
    """Create the elasticsearch index for records."""
    es.indices.delete(index='records', ignore=404)


def setup_app(app):
    """Set up the extension for the given app."""
    from invenio_base import signals
    from invenio_base.scripts.database import recreate, drop, create

    global es

    es = Elasticsearch(
        app.config.get('ES_HOSTS', None),
        connection_class=RequestsHttpConnection
    )

    signals.pre_command.connect(delete_index, sender=drop)
    signals.pre_command.connect(create_index, sender=create)
    signals.pre_command.connect(delete_index, sender=recreate)
    signals.pre_command.connect(create_index, sender=recreate)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2013, 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Exceptions for Restful."""


class RestfulError(Exception):

    """Generic Restful Error."""

    def __init__(self, error_msg=None, status_code=None):
        """Initialize error message and status code."""
        super(RestfulError, self).__init__()
        self.error_msg = error_msg or self.__class__.__name__()
        self.status_code = status_code or 400

    def __str__(self):
        """String representation of message."""
        return repr("Error message: %s , Status code: %s"
                    % (self.error_msg, self.status_code))


class InvalidPageError(RestfulError):

    """Error indicating an invalid page."""

    def __init__(self, error_msg="Invalid page", status_code=400):
        """Initialize default error message for invalid page error."""
        super(InvalidPageError, self).__init__(error_msg, status_code)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2011, 2012, 2013, 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Implement compressed column type."""

from sqlalchemy.types import LargeBinary, TypeDecorator

from invenio_utils.serializers import ZlibMarshal


class MarshalBinary(TypeDecorator):

    """Implement compressed column type."""

    impl = LargeBinary

    def __init__(self, default_value, force_type=None, *args, **kwargs):
        """Initialize default value and type."""
        super(MarshalBinary, self).__init__(*args, **kwargs)
        self.default_value = default_value
        self.force_type = force_type if force_type is not None else lambda x: x

    def process_bind_param(self, value, dialect):
        """Compress data in column."""
        if value is not None:
            value = ZlibMarshal.dumps(self.force_type(value))
            return value
        return value

    def process_result_value(self, value, dialect):
        """Load comressed data from column."""
        if value is not None:
            try:
                value = ZlibMarshal.loads(value)
            except:
                value = None
        return value if value is not None else \
            (self.default_value() if callable(self.default_value) else
             self.default_value)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2013, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""
Test unit for the miscutil/mailutils module.
"""

import os

import sys

from base64 import encodestring

from flask import current_app

from invenio_ext.email import send_email

from invenio_testing import InvenioTestCase

import pkg_resources

from six import StringIO, iteritems


class MailTestCase(InvenioTestCase):

    EMAIL_BACKEND = 'flask_email.backends.console.Mail'

    def setUp(self):
        super(MailTestCase, self).setUp()
        current_app.config['EMAIL_BACKEND'] = self.EMAIL_BACKEND
        self.__stdout = sys.stdout
        self.stream = sys.stdout = StringIO()

    def tearDown(self):
        del self.stream
        sys.stdout = self.__stdout
        del self.__stdout
        super(MailTestCase, self).tearDown()

    def flush_mailbox(self):
        self.stream = sys.stdout = StringIO()

    #def get_mailbox_content(self):
    #    messages = self.stream.getvalue().split('\n' + ('-' * 79) + '\n')
    #    return [message_from_string(m) for m in messages if m]


class TestMailUtils(MailTestCase):
    """
    mailutils TestSuite.
    """

    def test_console_send_email(self):
        """
        Test that the console backend can be pointed at an arbitrary stream.
        """
        msg_content = """Content-Type: text/plain; charset="utf-8"
MIME-Version: 1.0
Content-Transfer-Encoding: 7bit
Subject: Subject
From: from@example.com
To: to@example.com"""

        send_email('from@example.com', ['to@example.com'], subject='Subject',
                   content='Content')
        self.assertIn(msg_content, sys.stdout.getvalue())
        self.flush_mailbox()

        send_email('from@example.com', 'to@example.com', subject='Subject',
                   content='Content')
        self.assertIn(msg_content, sys.stdout.getvalue())
        self.flush_mailbox()

    def test_email_text_template(self):
        """
        Test email text template engine.
        """
        from invenio_ext.template import render_template_to_string

        contexts = {
            'ctx1': {'content': 'Content 1'},
            'ctx2': {'content': 'Content 2', 'header': 'Header 2'},
            'ctx3': {'content': 'Content 3', 'footer': 'Footer 3'},
            'ctx4': {'content': 'Content 4', 'header': 'Header 4', 'footer': 'Footer 4'}
        }

        msg_content = """Content-Type: text/plain; charset="utf-8"
MIME-Version: 1.0
Content-Transfer-Encoding: 7bit
Subject: %s
From: from@example.com
To: to@example.com"""

        for name, ctx in iteritems(contexts):
            msg = render_template_to_string('mail_text.tpl', **ctx)
            send_email('from@example.com', ['to@example.com'], subject=name,
                       **ctx)
            email = sys.stdout.getvalue()
            self.assertIn(msg_content % name, email)
            self.assertIn(msg, email)
            self.flush_mailbox()

    def test_email_html_template(self):
        """
        Test email html template engine.
        """
        from invenio_ext.template import render_template_to_string

        contexts = {
            'ctx1': {'html_content': '<b>Content 1</b>'},
            'ctx2': {'html_content': '<b>Content 2</b>',
                     'html_header': '<h1>Header 2</h1>'},
            'ctx3': {'html_content': '<b>Content 3</b>',
                     'html_footer': '<i>Footer 3</i>'},
            'ctx4': {'html_content': '<b>Content 4</b>',
                     'html_header': '<h1>Header 4</h1>',
                     'html_footer': '<i>Footer 4</i>'}
        }

        def strip_html_key(ctx):
            return dict(map(lambda (k, v): (k[5:], v), iteritems(ctx)))

        for name, ctx in iteritems(contexts):
            msg = render_template_to_string('mail_html.tpl',
                                            **strip_html_key(ctx))
            send_email('from@example.com', ['to@example.com'], subject=name,
                       content='Content Text', **ctx)
            email = sys.stdout.getvalue()
            self.assertIn('Content-Type: multipart/alternative;', email)
            self.assertIn('Content Text', email)
            self.assertIn(msg, email)
            self.flush_mailbox()

    def test_email_html_image(self):
        """
        Test sending html message with an image.
        """
        html_images = {
            'img1': pkg_resources.resource_filename(
                'invenio_base',
                os.path.join('static', 'img', 'journal_water_dog.gif')
            )
        }
        send_email('from@example.com', ['to@example.com'],
                   subject='Subject', content='Content Text',
                   html_content='<img src="cid:img1"/>',
                   html_images=html_images)
        email = sys.stdout.getvalue()
        self.assertIn('Content Text', email)
        self.assertIn('<img src="cid:img1"/>', email)
        with open(html_images['img1'], 'r') as f:
            self.assertIn(encodestring(f.read()), email)
        self.flush_mailbox()

    def test_sending_attachment(self):
        """
        Test sending email with an attachment.
        """
        attachments = [
            pkg_resources.resource_filename(
                'invenio_base',
                os.path.join('static', 'img', 'journal_header.png')
            )
        ]
        send_email('from@example.com', ['to@example.com'],
                   subject='Subject', content='Content Text',
                   attachments=attachments)
        email = sys.stdout.getvalue()
        self.assertIn('Content Text', email)
        # First attachemnt is image/png
        self.assertIn('Content-Type: image/png', email)
        for attachment in attachments:
            with open(attachment, 'r') as f:
                self.assertIn(encodestring(f.read()), email)
        self.flush_mailbox()

    def test_single_recipient(self):
        """
        Test that the email receivers are hidden.
        """
        msg_content = """Content-Type: text/plain; charset="utf-8"
MIME-Version: 1.0
Content-Transfer-Encoding: 7bit
Subject: Subject
From: from@example.com
To: to@example.com"""

        send_email('from@example.com', ['to@example.com'],
                   subject='Subject', content='Content')
        email = sys.stdout.getvalue()
        self.assertIn(msg_content, email)
        self.flush_mailbox()

        send_email('from@example.com', 'to@example.com',
                   subject='Subject', content='Content')
        email = sys.stdout.getvalue()
        self.assertIn(msg_content, email)
        self.flush_mailbox()

    def test_bbc_undisclosed_recipients(self):
        """
        Test that the email receivers are hidden.
        """
        msg_content = """Content-Type: text/plain; charset="utf-8"
MIME-Version: 1.0
Content-Transfer-Encoding: 7bit
Subject: Subject
From: from@example.com
To: Undisclosed.Recipients:"""

        send_email('from@example.com', ['to@example.com', 'too@example.com'],
                   subject='Subject', content='Content')
        email = sys.stdout.getvalue()
        self.assertIn(msg_content, email)
        self.assertNotIn('Bcc: to@example.com,too@example.com', email)
        self.flush_mailbox()

        send_email('from@example.com', 'to@example.com, too@example.com',
                   subject='Subject', content='Content')
        email = sys.stdout.getvalue()
        self.assertIn(msg_content, email)
        self.assertNotIn('Bcc: to@example.com,too@example.com', email)
        self.flush_mailbox()


class TestAdminMailBackend(MailTestCase):

    EMAIL_BACKEND = 'invenio_ext.email.backends.console_adminonly.Mail'
    ADMIN_MESSAGE = "This message would have been sent to the following recipients"

    def test_simple_email_header(self):
        """
        Test simple email header.
        """
        from invenio_base.globals import cfg
        from invenio_ext.template import render_template_to_string

        msg_content = """Content-Type: text/plain; charset="utf-8"
MIME-Version: 1.0
Content-Transfer-Encoding: 7bit
Subject: Subject
From: from@example.com
To: %s""" % (cfg['CFG_SITE_ADMIN_EMAIL'], )

        msg = render_template_to_string('mail_text.tpl', content='Content')

        self.flush_mailbox()
        send_email('from@example.com', ['to@example.com'], subject='Subject',
                   content='Content')
        email = self.stream.getvalue()
        self.assertIn(msg_content, email)
        self.assertIn(self.ADMIN_MESSAGE, email)
        self.assertNotIn('Bcc:', email)
        self.assertIn(msg, email)
        self.flush_mailbox()

        send_email('from@example.com', 'to@example.com', subject='Subject',
                   content='Content')
        email = self.stream.getvalue()
        self.assertIn(msg_content, email)
        self.assertIn(self.ADMIN_MESSAGE, email)
        self.assertNotIn('Bcc:', email)
        self.assertIn(msg, email)
        self.flush_mailbox()

    def test_cc_bcc_headers(self):
        """
        Test that no Cc and Bcc headers are sent.
        """
        from invenio_base.globals import cfg

        msg_content = """Content-Type: text/plain; charset="utf-8"
MIME-Version: 1.0
Content-Transfer-Encoding: 7bit
Subject: Subject
From: from@example.com
To: %s""" % (cfg['CFG_SITE_ADMIN_EMAIL'], )

        send_email('from@example.com', ['to@example.com', 'too@example.com'],
                   subject='Subject', content='Content')
        email = self.stream.getvalue()
        self.assertIn(msg_content, email)
        self.assertIn(self.ADMIN_MESSAGE, email)
        self.assertIn('to@example.com,too@example.com', email)
        self.assertNotIn('Bcc: to@example.com,too@example.com', email)
        self.flush_mailbox()

        send_email('from@example.com', 'to@example.com, too@example.com',
                   subject='Subject', content='Content')
        email = self.stream.getvalue()
        self.assertIn(msg_content, email)
        self.assertIn(self.ADMIN_MESSAGE, email)
        self.assertIn('to@example.com,too@example.com', email)
        self.assertNotIn('Bcc: to@example.com,too@example.com', email)
        self.flush_mailbox()



# -*- coding: utf-8 -*-
#
# This file is part of Invenio
# Copyright (C) 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Arxiv extension.

Arxiv extension is initialized like this:

>>> from flask import Flask
>>> from invenio_ext.arxiv import Arxiv
>>> app = Flask("myapp")
>>> ext = Arxiv(app=app)

Configuration Settings
----------------------
The details of the ArXiv URL and endpoint can be customized in the application
settings.

================= ============================================================
`ARXIV_API_URL`   The URL of ArXiV query API.
                  **Default:** `http://export.arxiv.org/oai2`
`ARXIV_ENDPOINT`  The name of Flask endpoint for new application route.
                  If the value is `False` (or `None`) the url rule is not
                  registered.  **Default:** `_arxiv.search`
`ARXIV_URL_RULE`  The URL for `ARXIV_ENPOINT` (i.e.
                  `url_for(current_app.config['ARXIV_ENDPOINT'])` is equal to
                  `current_app.config['ARXIV_ENDPOINT']`).
                  **Default:** `/arxiv/search`
================= ============================================================

"""

from __future__ import absolute_import

import requests

from flask import current_app, request, jsonify
from flask_login import login_required
from lxml.etree import fromstring

from invenio_utils.xmlhelpers import etree_to_dict

response_code = {'success': 200,
                 'notfound': 404,
                 'unsupported_versioning': 415,
                 'malformed': 422,
                 'multiplefound': 300}


class Arxiv(object):

    """Arxiv extension implementation.

    Initialization of the extension:

    >>> from flask import Flask
    >>> from invenio_ext.arxiv import Arxiv
    >>> app = Flask("myapp")
    >>> ext = Arxiv(app=app)

    or alternatively using the factory pattern:

    >>> app = Flask("myapp")
    >>> ext = Arxiv()
    >>> ext.init_app(app)
    """

    def __init__(self, app=None):
        """Initialize extension."""
        self.app = app
        if app is not None:
            self.init_app(app)

    def init_app(self, app):
        """Initialize a Flask application."""
        app.config.setdefault("ARXIV_API_URL",
                              "http://export.arxiv.org/oai2")
        app.config.setdefault("ARXIV_ENDPOINT", "_arxiv.search")
        app.config.setdefault("ARXIV_URL_RULE", "/arxiv/search")

        # Follow the Flask guidelines on usage of app.extensions
        if not hasattr(app, "extensions"):
            app.extensions = {}
        if "arxiv" in app.extensions:
            raise RuntimeError("Flask application already initialized")
        app.extensions["arxiv"] = self

        if app.config["ARXIV_ENDPOINT"]:
            app.add_url_rule(app.config["ARXIV_URL_RULE"],
                             app.config["ARXIV_ENDPOINT"],
                             login_required(self.search))

    def get_response(self, arxiv_id):
        """Get ArXiv response from the ``ARXIV_API_URL`` page."""
        response = requests.get(
            current_app.config["ARXIV_API_URL"],
            params=dict(
                verb="GetRecord",
                metadataPrefix="arXiv",
                identifier="oai:arXiv.org:{term}".format(
                    term=arxiv_id.strip()),
            ),
        )
        return response

    def get_json(self, arxiv_id):
        """Get ArXiv json data."""
        response = self.get_response(arxiv_id)
        data = etree_to_dict(fromstring(response.content))
        query = {}

        for d in data["OAI-PMH"]:
            query.update(dict(d.items()))
        del data["OAI-PMH"]

        if "error" in query:
            # Check if the ArXiv ID is malformed:
            if query["error"].startswith("Malformed"):
                query = {}
                data["status"] = "malformed"
            # Check if the ArXiv ID has version specified:
            elif 'versions' in query["error"]:
                query = {}
                data["status"] = "unsupported_versioning"
            # Otherwise, ArXiv ID was not found:
            else:
                query = {}
                data["status"] = "notfound"
        else:
            for d in query['GetRecord'][0][
                    'record'][1]['metadata'][0]['arXiv']:
                query.update(dict(d.items()))
            del query['GetRecord']
            data["status"] = "success"

        data["source"] = "arxiv"
        data["query"] = query

        return data

    def search(self, arxiv=None):
        """Search for given ArXiv ID."""
        arxiv = arxiv or request.args.get("arxiv")

        from invenio_records.utils import get_unique_record_json

        # query the database
        result = get_unique_record_json(
            "\"{0}{1}\"".format(self.app.config.get("ARXIV_SEARCH_PREFIX", ""), arxiv))
        if result["status"] == "notfound":
            # query arxiv
            result = self.get_json(arxiv)

        resp = jsonify(result)
        resp.status_code = response_code.get(result['status'],
                                             result['status'])
        return resp

__all__ = ("Arxiv", )



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2011, 2012, 2013, 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Implements various utility functions.

For example, session_manager used to handle commit/rollback:

    .. code-block:: python

        class SomeModel(db.Model):
            @session_manager
            def save(self):
                db.session.add(self)
"""

from __future__ import print_function

import base64
import os
import re
import sys

from intbitset import intbitset

import sqlalchemy
from sqlalchemy.exc import OperationalError
from sqlalchemy.ext.declarative import declared_attr
from sqlalchemy.orm import class_mapper, properties
from sqlalchemy.orm.collections import InstrumentedList, collection

first_cap_re = re.compile('(.)([A-Z][a-z]+)')
all_cap_re = re.compile('([a-z0-9])([A-Z])')


class TableNameMixin(object):

    """Define table name from class name."""

    @declared_attr
    def __tablename__(cls):
        """Generate table name as class name with lower starting character."""
        return cls.__name__[0].lower() + cls.__name__[1:]


class TableFromCamelNameMixin(object):

    """Define table name from class name."""

    @declared_attr
    def __tablename__(cls):
        """Generate underscored table name from camel cased class name."""
        s1 = first_cap_re.sub(r'\1_\2', cls.__name__)
        return all_cap_re.sub(r'\1_\2', s1).lower()


def get_model_type(ModelBase):
    """Return extended model type."""
    def getRelationships(self):
        """Return table relations."""
        retval = list()
        mapper = class_mapper(self)
        synonyms = dict()
        relationships = set()

        for prop in mapper.iterate_properties:
            if isinstance(prop, properties.SynonymProperty):
                synonyms[prop.name] = prop.key
                # dictionary <_userName, userName, userGroup, _userGroup>

            elif isinstance(prop, properties.RelationshipProperty):
                relationships.add(prop.key)
                # set with _userGroup, and rest of relationships

        for relationship in relationships:
            retval.append(synonyms[relationship])

        return retval

    def todict(self):
        """Convert model to dictionary."""
        def convert_datetime(value):
            try:
                return value.strftime("%Y-%m-%d %H:%M:%S")
            except Exception:
                return ''

        for c in self.__table__.columns:
            # NOTE This hack is not needed if you redefine types.TypeDecorator
            # for desired classes (Binary, LargeBinary, ...)

            value = getattr(self, c.name)
            if value is None:
                continue
            if isinstance(c.type, sqlalchemy.Binary):
                value = base64.encodestring(value)
            elif isinstance(c.type, sqlalchemy.DateTime):
                value = convert_datetime(value)
            elif isinstance(value, intbitset):
                value = value.tolist()
            yield(c.name, value)

    def fromdict(self, args):
        """Update instance from dictionary."""
        # NOTE Why not to do things simple ...
        self.__dict__.update(args)

        # for c in self.__table__.columns:
        #    name = str(c).split('.')[1]
        #    try:
        #        d = args[name]
        #    except Exception:
        #        continue
        #
        #    setattr(self, c.name, d)

    def __iter__(self):
        """Return an iterable that supports .next() for dict(sa_instance)."""
        return self.todict()

    ModelBase.todict = todict
    ModelBase.fromdict = fromdict
    ModelBase.__iter__ = __iter__
    ModelBase.__table_args__ = {}

    return ModelBase


def session_manager(orig_func):
    """Decorator to wrap function with the session.

    Useful to add to models functions that is meant to
    commit itself to DB when called.

    .. code-block:: python

        class SomeModel(db.Model):
            @session_manager
            def save(self):
                db.session.add(self)

    Now the session manager will handle committing and
    rollbacks on errors and re-raise.

    :param orig_func: original function
    :type orig_func: callable

    :return: decorated function.
    """
    from invenio_ext.sqlalchemy import db

    def new_func(self, *a, **k):
        """Wrapper function to manage DB session."""
        try:
            resp = orig_func(self, *a, **k)
            db.session.commit()
            return resp
        except Exception:
            db.session.rollback()
            raise

    return new_func


def test_sqla_connection():
    """Test connection with the database."""
    print("Test connection... \t", end="")

    try:
        from sqlalchemy import inspect
        from invenio_ext.sqlalchemy import db
        inspector = inspect(db.engine)
        inspector.get_table_names()
    except OperationalError as err:
        from invenio_utils.text import wrap_text_in_a_box
        from invenio_base.globals import cfg
        print(" [ERROR]")
        print(wrap_text_in_a_box("""\
DATABASE CONNECTIVITY ERROR:

%(errmsg)s.\n

Perhaps you need to set up database and connection rights?
If yes, then please execute:

console> inveniomanage database init --yes-i-know

The values printed above were detected from your
configuration. If they are not right, then please edit your
instance configuration file (invenio.cfg).


If the problem is of different nature, then please inspect
the above error message and fix the problem before continuing.""" % {
            'errmsg': err.args[0],
            'dbname': cfg['CFG_DATABASE_NAME'],
            'dbhost': cfg['CFG_DATABASE_HOST'],
            'dbport': cfg['CFG_DATABASE_PORT'],
            'dbuser': cfg['CFG_DATABASE_USER'],
            'dbpass': cfg['CFG_DATABASE_PASS'],
            'webhost': (cfg['CFG_DATABASE_HOST'] == 'localhost' and 'localhost' or
                        os.popen('hostname -f', 'r').read().strip()),
        }))
        sys.exit(1)
    print(" [OK]")


def test_sqla_utf8_chain():
    """Test insert and select of a UTF-8 string."""
    from sqlalchemy import Table, Column
    from sqlalchemy.types import CHAR, VARBINARY
    from sqlalchemy.ext.declarative import declarative_base

    from invenio_ext.sqlalchemy import db

    print("Test UTF-8 support... \t", end="")

    tmptable = "test__invenio__utf8"
    beta_in_utf8 = "β"  # Greek beta in UTF-8 is 0xCEB2

    Base = declarative_base()

    table = Table(tmptable, Base.metadata,
                  Column('x', CHAR(1)),
                  Column('y', VARBINARY(2)))

    if table.exists(bind=db.engine):
        table.drop(bind=db.engine)

    table.create(bind=db.engine)

    db.engine.execute(table.insert(), x=beta_in_utf8, y=beta_in_utf8)
    result = db.engine.execute(table.select().where(
        table.c.x == beta_in_utf8).where(table.c.y == beta_in_utf8))

    assert result.rowcount == 1

    for row in result:
        # Database is configured to always return Unicode.
        assert row['x'] == beta_in_utf8.decode('utf-8')
        assert row['y'] == beta_in_utf8

    table.drop(bind=db.engine)

    print(" [OK]")


class IntbitsetPickle(object):

    """Pickle implementation for intbitset."""

    def dumps(self, obj, protocol=None):
        """Dump intbitset to byte stream."""
        if obj is not None:
            return obj.fastdump()
        return intbitset([]).fastdump()

    def loads(self, obj):
        """Load byte stream to intbitset."""
        try:
            return intbitset(obj)
        except Exception:
            return intbitset()


def IntbitsetCmp(x, y):
    """Compare two intbitsets."""
    if x is None or y is None:
        return False
    else:
        return x == y


class OrderedList(InstrumentedList):

    """Implemented ordered instrumented list."""

    def append(self, item):
        """Append item."""
        if self:
            s = sorted(self, key=lambda obj: obj.score)
            item.score = s[-1].score + 1
        else:
            item.score = 1
        InstrumentedList.append(self, item)

    def set(self, item, index=0):
        """Set item."""
        if self:
            s = sorted(self, key=lambda obj: obj.score)
            if index >= len(s):
                item.score = s[-1].score + 1
            elif index < 0:
                item.score = s[0].score
                index = 0
            else:
                item.score = s[index].score + 1

            for i, it in enumerate(s[index:]):
                it.score = item.score + i + 1
                # if s[i+1].score more then break
        else:
            item.score = index
        InstrumentedList.append(self, item)

    def pop(self, item):
        """Pop item."""
        # FIXME
        if self:
            obj_list = sorted(self, key=lambda obj: obj.score)
            for i, it in enumerate(obj_list):
                if obj_list[i] == item:
                    return InstrumentedList.pop(self, i)


def attribute_multi_dict_collection(creator, key_attr, val_attr):
    """Define new attribute based mapping."""
    class MultiMappedCollection(dict):

        def __init__(self, data=None):
            self._data = data or {}

        @collection.appender
        def _append(self, obj):
            l = self._data.setdefault(key_attr(obj), [])
            l.append(obj)

        def __setitem__(self, key, value):
            self._append(creator(key, value))

        def __getitem__(self, key):
            return tuple(val_attr(obj) for obj in self._data[key])

        @collection.remover
        def _remove(self, obj):
            self._data[key_attr(obj)].remove(obj)

        @collection.iterator
        def _iterator(self):
            for objs in self._data.itervalues():
                for obj in objs:
                    yield obj

        def __repr__(self):
            return '%s(%r)' % (type(self).__name__, self._data)

    return MultiMappedCollection


def initialize_database_user(engine, database_name, database_user,
                             database_pass):
    """Grant user's privileges.

    :param engine: engine to use to execute queries
    :param database_name: database name
    :param database_user: the username that you want to init
    :param database_pass: password for the user
    """
    if engine.name == 'mysql':
        from MySQLdb import escape_string
        # create user and grant privileges
        for host in ['%%', 'localhost']:
            engine.execute((
                """GRANT ALL PRIVILEGES ON {0}.* """
                """TO {1}@'{2}' IDENTIFIED BY "{3}" """).format(
                    database_name,
                    database_user,
                    host,
                    escape_string(database_pass)
            ))
    elif engine.name == 'postgresql':
        # check if already exists
        res = engine.execute(
            """SELECT 1 FROM pg_roles WHERE rolname='{0}' """
            .format(database_user))
        # if already don't exists, create user
        if not res.first():
            from psycopg2.extensions import AsIs, adapt
            engine.execute(
                """CREATE USER %s WITH PASSWORD %s """,
                (AsIs(database_user), adapt(database_pass)))
        # grant privileges for user
        engine.execute(
            """grant all privileges on database """
            """ {0} to {1} """
            .format(
                database_name,
                database_user
            ))
    else:
        raise Exception((
            """Database engine %(engine)s not supported. """
            """You need to manually adds privileges to %(database_user)s """
            """in order to access to the database %(database_name)s.""") % {
                'engine': engine.name,
                'database_user': database_user,
                'database_name': database_name
        })



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.


"""Invenio 1.x style error handling.

Logs exceptions to database and sends emails. Works only in connection with
register_exception().

**Configuration**

======================== ======================================================
`LOGGING_LEGACY_LEVEL`   Log level threshold for handler. **Default:**
                         ``ERROR``.
======================== ======================================================
"""

from __future__ import absolute_import

import logging

from ..formatters import InvenioExceptionFormatter
from ..handlers import InvenioLegacyHandler


def setup_app(app):
    """Invenio 1.x log handler."""
    if not app.debug:
        app.config.setdefault('LOGGING_LEGACY_LEVEL', 'ERROR')

        handler = InvenioLegacyHandler()
        handler.setFormatter(InvenioExceptionFormatter())
        handler.setLevel(getattr(logging, app.config['LOGGING_LEGACY_LEVEL']))

        # Add handler to application logger
        app.logger.addHandler(handler)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""
Test unit for the miscutil/mailutils module.
"""

from __future__ import absolute_import

from flask_registry import ImportPathRegistry, RegistryError

from invenio_ext.registry import DictModuleAutoDiscoverySubRegistry
from invenio_testing import InvenioTestCase


class TestDictModuleAutoDiscoverySubRegistry(InvenioTestCase):
    def test_registration(self):
        r = self.app.extensions['registry']

        r['testpkgs'] = ImportPathRegistry(
            initial=['apps']
        )
        assert len(r['testpkgs']) == 1

        r['myns'] = \
            DictModuleAutoDiscoverySubRegistry(
                'last',
                keygetter=lambda k, v, new_v: k if k else v.__name__,
                app=self.app,
                registry_namespace='testpkgs'
            )
        assert len(r['myns']) == 1

        from apps.last import views
        assert r['myns']['apps.last.views'] == \
            views

        self.assertRaises(
            RegistryError,
            DictModuleAutoDiscoverySubRegistry,
            'last',
            app=self.app,
            registry_namespace='testpkgs'
        )

        # Register simple object
        class TestObject(object):
            pass

        r['myns'].register(TestObject)

        # Identical keys raises RegistryError
        self.assertRaises(
            RegistryError,
            r['myns'].register,
            TestObject
        )

        r['myns'].unregister('TestObject')
        assert 'TestObject' not in r['myns']

        r['myns']['mykey'] = TestObject
        assert TestObject == r['myns']['mykey']

        assert len(r['myns'].items()) == 2



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Utils related to Jinja templates."""

from flask import current_app


def render_macro_from_template(name, template, app=None, ctx=None):
    """Render macro with the given context.

    :param name: macro name.
    :type name: string.
    :param template: template name.
    :type template: string.
    :param app: Flask app.
    :type app: object.
    :param ctx: parameters of the macro.
    :type ctx: dict.
    :return: unicode string with rendered macro.
    """
    ctx = ctx or {}
    app = app or current_app
    tpl = app.jinja_env.get_template(template)
    macro = getattr(tpl.make_module(), name)
    return unicode(macro(**ctx))



# -*- coding: utf8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""
Flask-Collect custom storage for development mode.

It creates symbolic links to the real files so any changes to them will be
reflected.
"""

import os

from flask_collect.storage.base import BaseStorage


class Storage(BaseStorage):

    """Storage that creates symlinks to the resources."""

    def run(self):
        """Collect static from blueprints.

        Create the directory tree but will symlink all the files.
        """
        self.log("Collect static from blueprints")
        skipped, total = 0, 0
        for bp, f, o in self:
            destination = os.path.join(self.collect.static_root, o)
            destination_dir = os.path.dirname(destination)
            if not os.path.exists(destination_dir):
                os.makedirs(destination_dir)

            if not os.path.exists(destination):
                # the path is a link, but points to invalid location
                if os.path.islink(destination):
                    os.remove(destination)
                os.symlink(f, destination)
                self.log("{0}:{1} symbolink link created".format(bp.name, o))
            else:
                skipped += 1
            total += 1
        self.log("{0} of {1} files already present".format(skipped, total))
        self.log("Done collecting.")



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Define action class and registry."""

import six
from werkzeug.local import LocalProxy


class ActionsRegistry(type):

    """Action registry."""

    __actions_registry__ = []

    def __init__(cls, name, bases, dct):
        """Register cls to actions registry."""
        if not dct.get('__prototype__', False):
            cls.__actions_registry__.append(cls)
        super(ActionsRegistry, cls).__init__(name, bases, dct)

    @property
    def name(cls):
        """Return lowercased action class name."""
        return cls.__name__.lower()

    @property
    def description(cls):
        """Return stripped class documentation string."""
        return cls.__doc__.strip()

actions = LocalProxy(lambda: ActionsRegistry.__actions_registry__)
"""List of registered actions."""


@six.add_metaclass(ActionsRegistry)
class Action(object):

    """Default action description."""

    __prototype__ = True  # do not register this class

    allowedkeywords = []

    optional = False



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2013, 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Custom hooks and hacks needed for Flask-Babel."""

import sys

from contextlib import contextmanager

from babel import support

from flask import _request_ctx_stack, current_app, g

from flask_babel import Babel, gettext

from invenio_utils.datastructures import LaziestDict

import pkg_resources

import six

from .errors import NoCompiledTranslationError
from .filters import filter_format_date, filter_format_datetime, \
    filter_format_time, filter_format_timedelta, filter_to_user_timezone, \
    filter_to_utc
from .selectors import get_locale, get_timezone

babel = Babel()


@contextmanager
def set_locale(ln):
    """Set Babel localization in request context."""
    ctx = _request_ctx_stack.top
    locale = getattr(ctx, 'babel_locale', None)
    setattr(ctx, 'babel_locale', ln)
    yield
    setattr(ctx, 'babel_locale', locale)


def get_translation(locale):
    """Generate translation for given language."""
    translations = support.Translations.load(None, [locale])
    # NOTE that packages have to be loaded in reversed order!
    for plugin in reversed(current_app.extensions['registry']['packages']):
        if not pkg_resources.resource_isdir(plugin, 'translations'):
            continue
        dirname = pkg_resources.resource_filename(plugin, 'translations')
        if not hasattr(translations, 'merge'):
            translations = support.Translations.load(dirname, [locale])
        else:
            try:
                translations.merge(
                    support.Translations.load(dirname, [locale]))
            except AttributeError:
                # translations is probably support.NullTranslations
                six.reraise(NoCompiledTranslationError,
                            "Compiled translations seems to be missing",
                            sys.exc_info()[2])
    return translations

# Lazy translation cache.
TRANSLATIONS = LaziestDict(get_translation)


def get_translations():
    """Return the correct gettext translations for this request."""
    ctx = _request_ctx_stack.top
    if ctx is None:
        return None
    translations = getattr(ctx, 'babel_translations', None)
    if translations is None:
        locale = get_locale()
        translations = TRANSLATIONS.get(locale)
        ctx.babel_translations = translations
    return translations


def set_translations():
    """Add under ``g._`` an already configured internationalization function.

    Translations will be returned as unicode objects.
    """
    # Well, let's make it global now
    g.ln = get_locale()
    g._ = gettext


def get_lazystring_encoder(app):
    """Add a custom JSONEncoder that handles lazy strings from Babel."""
    from speaklater import _LazyString

    class JSONEncoder(app.json_encoder):

        def default(self, o):
            if isinstance(o, _LazyString):
                return six.text_type(o)

            return super(JSONEncoder, self).default(o)

    return JSONEncoder


def setup_app(app):
    """Setup Babel extension.

    Replaces implementation of ``get_translations`` in ``flask_babel``.
    """
    import flask_babel
    flask_babel.get_translations = get_translations

    babel.init_app(app)
    babel.localeselector(get_locale)
    babel.timezoneselector(get_timezone)

    app.json_encoder = get_lazystring_encoder(app)
    app.before_request(set_translations)

    # Register template filters for date formatting
    app.add_template_filter(filter_format_date, name="formatdate")
    app.add_template_filter(filter_format_datetime, name="formatdatetime")
    app.add_template_filter(filter_format_time, name="formattime")
    app.add_template_filter(filter_format_timedelta, name="formattimedelta")
    app.add_template_filter(filter_to_utc, name="toutc")
    app.add_template_filter(filter_to_user_timezone, name="tousertimezone")

    return app



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

from __future__ import absolute_import, print_function

import logging

from .wrappers import get_pretty_traceback


class InvenioExceptionFormatter(logging.Formatter):
    """
    Invenio exception formatter
    """

    def __init__(self, fmt=None, datefmt=None, remove_trailing_linefeed=False):
        self.remove_trailing_linefeed = remove_trailing_linefeed
        logging.Formatter.__init__(self, fmt=fmt, datefmt=datefmt)

    def format(self, record):
        """ Format log record """
        # Set by invenio_ext.logging.wrappers.register_exception
        if hasattr(record, 'invenio_register_exception'):
            extra = record.invenio_register_exception

            exc_info = record.exc_info
            req = extra['req']
            prefix = extra['prefix']
            suffix = extra['suffix']

            output = get_pretty_traceback(
                req=req, exc_info=exc_info, skip_frames=2
            )

            if output:
                if prefix:
                    output = prefix + '\n' + output
                if suffix:
                    output = output + suffix

                if self.remove_trailing_linefeed and output.endswith('\n'):
                    output = output[:-1]

                return output
        return logging.Formatter.format(self, record)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2013, 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Addtional registries for Flask-Registry."""

from werkzeug.utils import import_string, find_modules
from flask_registry import ModuleAutoDiscoveryRegistry
from flask_registry import RegistryError


class ModuleAutoDiscoverySubRegistry(ModuleAutoDiscoveryRegistry):

    def _discover_module(self, pkg):
        import_str = pkg + '.' + self.module_name

        blacklist = self.app.config.get(
            '%s_%s_EXCLUDE' % (self.cfg_var_prefix, self.module_name.upper()),
            []
        )

        try:
            import_string(import_str, silent=self.silent)
        except ImportError as e:  # pylint: disable=C0103
            self._handle_importerror(e, pkg, import_str)
            return
        except SyntaxError as e:
            self._handle_syntaxerror(e, pkg, import_str)
            return

        for m in find_modules(import_str):
            if m in blacklist:  # Exclude specific package
                continue
            try:
                module = import_string(m, silent=self.silent)
                if module is not None:
                    self.register(module)
            except ImportError as e:  # pylint: disable=C0103
                self._handle_importerror(e, import_str, m)
            except SyntaxError as e:
                self._handle_syntaxerror(e, import_str, m)


class DictModuleAutoDiscoverySubRegistry(ModuleAutoDiscoverySubRegistry):
    """
    ModuleAutoDiscoverySubRegistry that behaves like a dictionary.

    You should either provide a function to the ``keygetter`` keyword argument
    or subclass and override the ``keygetter()`` method for determining the
    key of values in the dictionary. Likewise provide ``valuegetter`` in the
    same way to determine the registered value.
    """

    def __init__(self, *args, **kwargs):
        self._registry = {}
        self.registry = None  # Kill internal registry from ListRegistry
        self._keygetter = kwargs.pop('keygetter', None)
        self._valuegetter = kwargs.pop('valuegetter', None)
        super(DictModuleAutoDiscoverySubRegistry, self).__init__(
            *args, **kwargs
        )

    def keygetter(self, key, original_value, new_value):
        """
        Method used to compute the key for a value being registered.

        Either use the ``keygetter`` argument during initialization or subclass
        and override this method to customize the behaviour.

        :param key: Key if provided by the user. Defaults to None.
        :param original_value: Unmodified value being registered.
        :param new_value: Modified value being registered.
        """
        if self._keygetter:
            return self._keygetter(key, original_value, new_value)
        raise RegistryError("Please provide a key or keygetter function.")

    def valuegetter(self, value):
        """
        Method use to modify the value being registered. By default it just
        returns the same object.
        """
        return self._valuegetter(value) if self._valuegetter else value

    def register(self, value, key=None):  # pylint: disable=W0221
        """
        :param value: Value being registered.
        :param key: Key, if provided by the user.
        """
        new_value = self.valuegetter(value)
        key = self.keygetter(key, value, new_value)

        if key in self._registry:
            raise RegistryError("Key %s already registered." % key)

        if new_value:
            self._registry[key] = new_value

    def unregister(self, key):  # pylint: disable=W0221
        """
        Unregister an object under a given key. Raises ``KeyError`` in case
        the given key doesn't exists.
        """
        del self[key]

    def __iter__(self):
        # pylint: disable=R0921
        return self._registry.__iter__()

    def __len__(self):
        return self._registry.__len__()

    def __contains__(self, item):
        return self._registry.__contains__(item)

    def __getitem__(self, key):
        return self._registry[key]

    def __setitem__(self, key, value):
        return self.register(value, key=key)

    def __delitem__(self, key):
        return self._registry.__delitem__(key)

    def items(self):
        """
        Get list of key/value pairs.
        """
        return self._registry.items()

    def keys(self):
        """
        Get list of keys.
        """
        return self._registry.keys()

    def values(self):
        """
        Get list of values.
        """
        return self._registry.values()

    def iteritems(self):
        """
        Get iterator over list of key/value pairs.
        """
        return self._registry.iteritems()

    def get(self, key, default=None):
        """
        Get value of key.

        :param key: key to fetch
        :param default: default if key not found (optional)

        :return: value of given key in dict
        """
        return self._registry.get(key, default)
